package com.bocloud.mcs.collector.fusioncloud;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cms.entity.MonitorAuth;
import com.bocloud.common.encrypt.AESEncrptor;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.ListTool;
import com.bocloud.ims.entity.CloudVendor;
import com.bocloud.ims.enums.CloudProvider;
import com.bocloud.mcs.collector.BocloudCollector;
import com.bocloud.mcs.collector.fusioncloud.core.FusionCloudClient;
import com.bocloud.mcs.service.internal.CspCloudVendorService;
import com.bocloud.service.core.CurrentService;
import io.prometheus.client.GaugeMetricFamily;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.scheduling.annotation.Async;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;
import org.springframework.util.Assert;

import java.util.*;
import java.util.concurrent.ConcurrentHashMap;

/**
 * fusioncloud collector
 *
 * @author zhangyf
 * @since 2019-10-12
 */
@Component
public class FusionCloudHostCollector extends BocloudCollector implements InitializingBean {

    Logger logger = LoggerFactory.getLogger(FusionCloudHostCollector.class);

    final
    CspCloudVendorService cspCloudVendorService;

    @Autowired
    private CurrentService currentService;

    /**
     * 本地缓存
     */
    Map<String, Object> cache = new HashMap<>();
    List<Long> indicatorIds;
    JSONObject indicators;

    @Value("${collector.fusioncloud.host.enable:true}")
    boolean enable;
    private String prefix = "bocloud_fusioncloud_host_";

    @Autowired
    public FusionCloudHostCollector(CspCloudVendorService cspCloudVendorService) {
        this.cspCloudVendorService = cspCloudVendorService;
    }

    @Scheduled(initialDelay = 1000L, fixedRate = 60000)
    @Async
    @Override
    public void scheduledCollect() {
        if(!enable){
            return;
        }
        if (!currentService.isLeader()) {
            logger.info(this.getClass().getName() + "当前节点是否是主节点:" + currentService.isLeader());
            return;
        }
        Date now = new Date();
        logger.info("collect fusioncloud host metrics start...");
        GaugeMetricFamily collectTimeUse = new GaugeMetricFamily(prefix + "collect_time_use", "help",
                Collections.singletonList("type"));
        BocloudResult result = cmpCloudVendorService.list(CloudProvider.FUSIONCLOUD);
        Assert.isTrue(result.isSuccess(), "get cloud vendor list failed , error:" + result.getMessage());
        JSONObject data = (JSONObject) result.getData();
        List<CloudVendor> vendors = data.getJSONArray(Common.ROWS).toJavaList(CloudVendor.class);
        result = cmpCloudVendorService.list(CloudProvider.MANAGEONE);
        Assert.isTrue(result.isSuccess(), "get cloud vendor list failed , error:" + result.getMessage());
        data = (JSONObject) result.getData();
        vendors.addAll(data.getJSONArray(Common.ROWS).toJavaList(CloudVendor.class));

        Map<String, GaugeMetricFamily> map = new ConcurrentHashMap<>();
        for (CloudVendor vendor : vendors) {
            try {
                FusionCloudClient client = new FusionCloudClient();
                try {
                    // 获取fusioncloud运维侧地址及认证信息
                    BocloudResult authResult = cspCloudVendorService.detailVendorAuth(vendor.getId());
                    Assert.isTrue(authResult.isSuccess(), "get vendor operations auth failed , error:" + authResult.getMessage());
                    MonitorAuth monitorAuth = JSONObject.parseObject(JSONObject.toJSONString(authResult.getData()), MonitorAuth.class);
                    Assert.notNull(monitorAuth, "vendor [" + vendor.getName() + "] not found auth config info");
                    String password = new AESEncrptor().decrypt(monitorAuth.getPassword(), null);
                    client.auth(monitorAuth.getAddress(), monitorAuth.getUsername(), password);
                } catch (Exception e) {
                    logger.error("auth error ", e);
                    continue;
                }
                List<JSONObject> objects = client.listPhysicalHosts();//宿主机集合
                if (ListTool.isEmpty(indicatorIds)) {
                    indicatorIds = client.listIndicatorIds(client.getHostObjTypeId()); // 支持的监控指标集合IndicatorIds
                }
                if (indicators == null || indicators.size() == 0) {
                    indicators = client.listIndicators(indicatorIds); // 指标的数字和描述信息
                }
                client.collectMetrics(vendor, objects, indicatorIds, indicators, client.getHostObjTypeId(), map, prefix);
            } catch (RuntimeException e) {
                logger.error("invoke fusioncloud host get metrics error,", e);
            }
        }
        List<MetricFamilySamples> mfs = new ArrayList<>(map.values());
        collectTimeUse.addMetric(Collections.singletonList("fusioncloud_host"), (System.currentTimeMillis() - now.getTime()));
        mfs.add(collectTimeUse);
        cache.put(TIME, new Date());
        cache.put(DATA, mfs);
        logger.info("collect fusioncloud host metrics end! use " + (System.currentTimeMillis() - now.getTime()) / 1000 + "s");
    }

    @Override
    public void afterPropertiesSet() {
        register();
    }

    @Override
    public Map<String, Object> getCache() {
        return this.cache;
    }

}
