package com.bocloud.mcs.collector.fusioncloud.core;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.enums.PostDataFormat;
import com.bocloud.common.http.HttpClient;
import com.bocloud.common.model.Result;
import com.bocloud.ims.entity.CloudVendor;
import io.prometheus.client.GaugeMetricFamily;
import org.springframework.util.Assert;

import java.util.*;
import java.util.stream.Collectors;

/**
 * FusionCloud
 *
 * @author zhangyf
 * @since 2019-10-17
 */
public class FusionCloudClient {

    private HttpClient client = new HttpClient(PostDataFormat.RAW);
    private String endpoint;
    private String token;
    /**
     * roaRand
     */
    private String roaRand;

    /**
     * 运维侧权限认证
     *
     * @param endpoint 端点
     * @param user     用户名
     * @param password 密码
     */
    public void auth(String endpoint, String user, String password) {
        this.endpoint = endpoint;
        String url = endpoint + "/rest/plat/smapp/v1/oauth/token";
        Map<String, Object> params = new HashMap<>(4);
        params.put("grantType", "password");
        params.put("userName", user);
        params.put("value", password);

        Map<String, Object> header = new HashMap<>(3);
        header.put("accept", "application/json");
        header.put("content-type", "application/json");
        Result result = client.put(header, params, url);
        Assert.isTrue(result.isSuccess(), result.getMessage());
        JSONObject content = JSONObject.parseObject(result.getMessage());
        this.token = content.getString("accessSession");
        this.roaRand = content.getString("roaRand");
    }

    public void collectMetrics(CloudVendor vendor, List<JSONObject> objects, List<Long> indicatorIds, JSONObject indicators,
                               Long objTypeId, Map<String, GaugeMetricFamily> map, String prefix) {
        List<String> objIds = Optional.ofNullable(objects).orElse(new ArrayList<>()).stream().map(vm -> vm.getString("resId")).collect(Collectors.toList());
        Map<String, JSONObject> objIdNativeIdMap = new HashMap<>(); //resId，nativeId(openstack里资源的id)
        Optional.ofNullable(objects).orElse(new ArrayList<>()).forEach(object -> objIdNativeIdMap.put(object.getString("resId"), object));

        JSONObject metrics = this.metrics(objIds, objTypeId, indicatorIds);
        objIds.forEach(objId -> {
            JSONObject metric = metrics.getJSONObject(objId);
            Optional.ofNullable(metric).orElse(new JSONObject()).forEach((key, value) -> {
                JSONObject indicator = indicators.getJSONObject(key);
                if (!Optional.ofNullable(map.get(key)).isPresent()) {
                    GaugeMetricFamily gaugeMetricFamily = new GaugeMetricFamily(prefix + indicator.getString("indicator_name"),
                            indicator.getString("zh_cn"), Arrays.asList("vendorId", "id", "name"));
                    map.put(key, gaugeMetricFamily);
                }
                GaugeMetricFamily gaugeMetricFamily = map.get(key);
                gaugeMetricFamily.addMetric(Arrays.asList(String.valueOf(vendor.getId()), objIdNativeIdMap.get(objId).getString("nativeId"), objIdNativeIdMap.get(objId).getString("name")), metric.getJSONObject(key).getDoubleValue("last_collect_data"));
            });
        });
    }

    /**
     * 获取宿主机列表
     *
     * @return 宿主机列表
     */
    public List<JSONObject> listPhysicalHosts() {
        int pageNo = 1;
        int pageSize = 1000;
        List<JSONObject> list = null;
        while (true) {
            String url = this.endpoint + "/rest/cmdb/v1/instances/SYS_PhysicalHost";
            Map<String, Object> params = new HashMap<>();
            params.put("pageSize", pageSize);
            params.put("pageNo", pageNo++);
            params.put("contentSelector", JSONObject.toJSONString(Arrays.asList("resId", "nativeId", "name")));
            Result result = client.get(defaultHeader(), params, url);
            JSONObject content = JSONObject.parseObject(result.getMessage());
            Assert.isTrue(result.isSuccess(), result.getMessage());
            if (list == null) {
                list = new ArrayList<>(content.getIntValue("totalNum"));
            }
            list.addAll(JSONObject.parseArray(content.getString("objList"), JSONObject.class));
            if (content.getIntValue("totalPageNo") == content.getIntValue("currentPage")) {
                break;
            }
        }
        return list;
    }

    /**
     * 获取虚拟机列表
     *
     * @return 虚拟机列表
     */
    public List<JSONObject> listCloudVms() {
        return listTenantResource("CLOUD_VM");
    }


    /**
     * 获取云硬盘列表
     *
     * @return 云硬盘列表
     */
    public List<JSONObject> listCloudVolumes() {
        return listTenantResource("CLOUD_VOLUME");
    }

    /**
     * 租户资源查询
     *
     * @param className 资源类型，和《租户资源北向模型》中的资源类型与视图类型保持一致。
     * @return
     */
    private List<JSONObject> listTenantResource(String className) {
        int limit = 1000;
        int offset = 0;
        List<JSONObject> list = null;
        while (true) {
            String url = this.endpoint + "/rest/tenant-resource/v1/tenant/resources/" + className;
            JSONObject params = new JSONObject();
            params.put("limit", limit);
            params.put("offset", offset);
            Result result = client.get(defaultHeader(), params, url);
            JSONObject content = JSONObject.parseObject(result.getMessage());
            Assert.isTrue(result.isSuccess(), result.getMessage());
            if (list == null) {
                list = new ArrayList<>(content.getIntValue("totalNum"));
            }
            list.addAll(JSONObject.parseArray(content.getString("objList"), JSONObject.class));
            offset = limit * content.getIntValue("currentPage");
            if (content.getIntValue("totalPageNo") == content.getIntValue("currentPage")) {
                break;
            }
        }
        return list;
    }

    /**
     * 获取最新性能数据
     *
     * @param objIds       资源resId集合
     * @param objTypeId    资源对象类型Id
     * @param indicatorIds 监控指标集合IndicatorIds
     * @return 最新性能数据
     */
    public JSONObject metrics(List<String> objIds, Long objTypeId, List<Long> indicatorIds) {
        JSONObject jsonObject = new JSONObject();
        String url = this.endpoint + "/rest/performance/v1/data-svc/latest-data/action/query";
        Optional.ofNullable(objIds).orElse(new ArrayList<>()).forEach(objId -> {
            JSONObject params = new JSONObject();
            params.put("obj_type_id", objTypeId);
            params.put("indicator_ids", indicatorIds);
            params.put("obj_ids", Collections.singletonList(objId));
            Result result = client.post(defaultHeader(), params, url);
            Assert.isTrue(result.isSuccess(), result.getMessage());
            jsonObject.putAll(JSONObject.parseObject(result.getMessage()).getJSONObject("data"));
        });
        return jsonObject;
    }

    /**
     * 查询指标的数字和描述信息
     *
     * @return 指标的数字和描述信息
     */
    public JSONObject listIndicators(List<Long> indicatorIds) {
        String url = this.endpoint + "/rest/performance/v1/mgr-svc/indicators";
        Result result = client.post(defaultHeader(), indicatorIds, url);
        Assert.isTrue(result.isSuccess(), result.getMessage());
        return JSONObject.parseObject(result.getMessage()).getJSONObject("data");
    }

    /**
     * 获取objType支持的监控指标集合IndicatorIds
     *
     * @param objTypeId objTypeId
     * @return indicatorIds
     */
    public List<Long> listIndicatorIds(Long objTypeId) {
        String url = this.endpoint + "/rest/performance/v1/mgr-svc/obj-types/" + objTypeId + "/indicators";
        Result result = client.get(defaultHeader(), null, url);
        Assert.isTrue(result.isSuccess(), result.getMessage());
        return JSONObject.parseObject(result.getMessage()).getJSONObject("data").getJSONArray("indicator_ids").toJavaList(Long.class);
    }

    /**
     * 默认header，包含认证信息
     *
     * @return header
     */
    private Map<String, Object> defaultHeader() {
        Map<String, Object> header = new HashMap<>();
        header.put("Accept", "application/json");
        header.put("Content-Type", "application/json;charset=UTF-8");
        header.put("X-Auth-Token", this.token);
        header.put("roaRand", this.roaRand);
        return header;
    }

    /**
     * 获取虚拟机对象类型对应的id(objTypeId)
     *
     * @return 虚拟机类型id
     */
    public Long getVmObjTypeId() {
        return 562958543355904L;
    }

    /**
     * 获取宿主机对象类型对应的id(objTypeId)
     *
     * @return 宿主机类型id
     */
    public Long getHostObjTypeId() {
        return 1407379178520576L;
    }

    /**
     * 获取云硬盘对象类型对应的id(objTypeId)
     *
     * @return 云硬盘类型id
     */
    public Long getEvsDiskObjTypeId() {
        return 562967133290496L;
    }

}
