package com.bocloud.mcs.collector.hmc;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.encrypt.DESEncrptor;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.utils.Common;
import com.bocloud.ims.entity.CloudVendor;
import com.bocloud.ims.entity.HmcPartition;
import com.bocloud.ims.entity.HmcServer;
import com.bocloud.ims.enums.CloudProvider;
import com.bocloud.mcs.collector.BocloudCollector;
import com.bocloud.mcs.collector.hmc.utils.MathUtils;
import com.bocloud.mcs.common.Constants;
import com.bocloud.mcs.service.internal.CmpCloudVendorService;
import com.bocloud.service.core.CurrentService;
import com.google.common.util.concurrent.ThreadFactoryBuilder;
import io.prometheus.client.GaugeMetricFamily;
import org.apache.commons.lang3.BooleanUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.time.DateUtils;
import org.apache.http.config.Registry;
import org.apache.http.config.RegistryBuilder;
import org.apache.http.conn.socket.ConnectionSocketFactory;
import org.apache.http.conn.socket.PlainConnectionSocketFactory;
import org.apache.http.conn.ssl.NoopHostnameVerifier;
import org.apache.http.conn.ssl.SSLConnectionSocketFactory;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.impl.conn.PoolingHttpClientConnectionManager;
import org.apache.http.ssl.SSLContextBuilder;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.*;
import org.springframework.http.client.HttpComponentsClientHttpRequestFactory;
import org.springframework.scheduling.annotation.Async;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.util.MultiValueMap;
import org.springframework.web.client.RestTemplate;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

import javax.xml.XMLConstants;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.security.KeyManagementException;
import java.security.KeyStoreException;
import java.security.NoSuchAlgorithmException;
import java.security.cert.X509Certificate;
import java.text.SimpleDateFormat;
import java.time.ZoneId;
import java.time.ZonedDateTime;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.concurrent.*;
import java.util.stream.Collectors;

@Component
public class HmcCollector extends BocloudCollector implements InitializingBean {

    private final Logger logger = LoggerFactory.getLogger(HmcCollector.class);
    private final Map<String, Object> cache = new HashMap<>();
    private final RestTemplate restTemplate;
    private final int threadPoolSize;

    @Value("${collector.hmc.enable:true}")
    private boolean enable;
    @Value("${collector.hmc.preference:LongTermMonitorEnabled,AggregationEnabled}")
    private String preference;
    @Autowired
    private CmpCloudVendorService cmpCloudVendorService;
    @Autowired
    private CurrentService currentService;

    private final String loginRequest =
            "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"yes\"?>\n" +
                    "<LogonRequest xmlns=\"http://www.ibm.com/xmlns/systems/power/firmware/web/mc/2012_10/\" schemaVersion=\"V1_1_0\">\n" +
                    "  <Metadata><Atom/></Metadata>\n" +
                    "  <UserID kb=\"CUR\" kxe=\"false\">%s</UserID>\n" +
                    "  <Password kb=\"CUR\" kxe=\"false\">%s</Password>\n" +
                    "</LogonRequest>";
    private final String loginUri = "/rest/api/web/Logon";
    private final String preferencesUri = "/rest/api/pcm/ManagedSystem/%s/preferences";
    private final String serverUri = "/rest/api/uom/ManagedSystem";
    private final String viosUri = "/rest/api/uom/ManagedSystem/%s/VirtualIOServer";
    private final String ltmUri = "/rest/api/pcm/ManagedSystem/%s/RawMetrics/LongTermMonitor?StartTS=%s";
    private final String procUri = "/rest/api/pcm/ManagedSystem/%s/ProcessedMetrics?StartTS=%s";
    private final List<String> labels = Arrays.asList("vendorId", "instanceUuid", "instanceName", "parentUuid", "parentName");

    private final ExecutorService executorService;

    @Autowired
    public HmcCollector(@Value("${collector.hmc.threadPoolSize:0}") int threadPoolSize) throws KeyStoreException, NoSuchAlgorithmException, KeyManagementException {
        int processorCount = Runtime.getRuntime().availableProcessors();
        this.threadPoolSize = threadPoolSize > 0 ? threadPoolSize : processorCount;
        if (this.threadPoolSize > processorCount * 3) {
            logger.warn("You will create a thread pool with size {}, current processor count is {}, still do it",
                threadPoolSize, processorCount);
        }
        ThreadFactory nameThreadFactory = new ThreadFactoryBuilder().setNameFormat("hmc-pool-%d").build();
        executorService = new ThreadPoolExecutor(this.threadPoolSize, this.threadPoolSize,
                0L, TimeUnit.MILLISECONDS,
                new LinkedBlockingQueue<>(), nameThreadFactory);

        HttpComponentsClientHttpRequestFactory factory = new HttpComponentsClientHttpRequestFactory();
        SSLContextBuilder builder = new SSLContextBuilder();
        builder.loadTrustMaterial(null, (X509Certificate[] x509Certificates, String s) -> true);
        SSLConnectionSocketFactory socketFactory = new SSLConnectionSocketFactory(builder.build(), NoopHostnameVerifier.INSTANCE);
        Registry<ConnectionSocketFactory> registry = RegistryBuilder.<ConnectionSocketFactory>create()
                .register("http", new PlainConnectionSocketFactory())
                .register("https", socketFactory).build();
        PoolingHttpClientConnectionManager phccm = new PoolingHttpClientConnectionManager(registry);
        phccm.setMaxTotal(200);
        CloseableHttpClient httpClient = HttpClients.custom().setSSLSocketFactory(socketFactory).setConnectionManager(phccm).setConnectionManagerShared(true).build();
        factory.setHttpClient(httpClient);

        restTemplate = new RestTemplate(factory);
    }

    private String login(String host, String username, String password) throws ParserConfigurationException, IOException, SAXException {
        HttpHeaders httpHeaders = new HttpHeaders();
        httpHeaders.set("Content-Type", "application/vnd.ibm.powervm.web+xml");

        String body = String.format(loginRequest, username, password);

        HttpEntity<String> httpEntity = new HttpEntity<String>(body, httpHeaders);
        long start = System.currentTimeMillis();
        ResponseEntity<byte[]> responseEntity = restTemplate.exchange(
                host + loginUri, HttpMethod.PUT, httpEntity, byte[].class
        );
        logger.debug("Login to HMC[" + host + "] end, use: " + (System.currentTimeMillis() - start) / 1000D + "s");
        DocumentBuilderFactory documentBuilderFactory = DocumentBuilderFactory.newInstance();
        documentBuilderFactory.setExpandEntityReferences(false);
        documentBuilderFactory.setFeature(XMLConstants.FEATURE_SECURE_PROCESSING, true);
        DocumentBuilder documentBuilder = documentBuilderFactory.newDocumentBuilder();
        Document document = documentBuilder.parse(new ByteArrayInputStream(responseEntity.getBody()));
        return document.getDocumentElement().getElementsByTagName("X-API-Session").item(0).getTextContent();
    }

    private void logout(String host, String sessionId) {
        HttpHeaders httpHeaders = new HttpHeaders();
        httpHeaders.set("X-Transaction-ID", sessionId);

        HttpEntity<String> httpEntity = new HttpEntity<>("", httpHeaders);
        long start = System.currentTimeMillis();
        ResponseEntity<byte[]> responseEntity = restTemplate.exchange(
                host + loginUri, HttpMethod.DELETE, httpEntity, byte[].class
        );
        logger.debug("Logout to HMC[" + host + "] end, use: " + (System.currentTimeMillis() - start) / 1000D + "s");
    }

    private List<HmcServer> getServers(String host, String sessionId) throws ParserConfigurationException, IOException, SAXException {
        HttpHeaders httpHeaders = new HttpHeaders();
        httpHeaders.set("X-Transaction-ID", sessionId);

        HttpEntity<String> httpEntity = new HttpEntity<>("", httpHeaders);
        long start = System.currentTimeMillis();
        ResponseEntity<byte[]> responseEntity = restTemplate.exchange(
                host + serverUri, HttpMethod.GET, httpEntity, byte[].class
        );
        logger.debug("Get HMC[" + host + "] server end, use: " + (System.currentTimeMillis() - start) / 1000D + "s");
        List<HmcServer> servers = new ArrayList<>();
        DocumentBuilderFactory documentBuilderFactory = DocumentBuilderFactory.newInstance();
        documentBuilderFactory.setExpandEntityReferences(false);
        documentBuilderFactory.setFeature(XMLConstants.FEATURE_SECURE_PROCESSING, true);
        DocumentBuilder documentBuilder = documentBuilderFactory.newDocumentBuilder();
        Document document = documentBuilder.parse(new ByteArrayInputStream(responseEntity.getBody()));
        // 查询主机接口返回数据中,feed.entry.id为主机id
        NodeList entries = document.getDocumentElement().getElementsByTagName("entry");
        for (int i = 0; i < entries.getLength(); i++) {
            Element entry = (Element) entries.item(i);
            String serverId = entry.getElementsByTagName("id").item(0).getTextContent();

            Element content = (Element) entry.getElementsByTagName("content").item(0);
            Element managedSystem = (Element) content.getElementsByTagName("ManagedSystem:ManagedSystem").item(0);
            Element systemName = (Element) managedSystem.getElementsByTagName("SystemName").item(0);
            String serverName = systemName.getTextContent();

            HmcServer server = new HmcServer();
            server.setUuid(serverId);
            server.setName(serverName);
            servers.add(server);
        }
        return servers;
    }

    private String setPreferences(String host, String serverId, String sessionId) throws ParserConfigurationException, IOException, SAXException, TransformerException {
        if (StringUtils.isEmpty(preference)) {
            return null;
        }
        String[] autoPreferences = preference.split(",");

        HttpHeaders httpHeaders = new HttpHeaders();
        httpHeaders.set("X-Transaction-ID", sessionId);
        httpHeaders.setContentType(MediaType.APPLICATION_XML);

        HttpEntity<String> httpEntityGet = new HttpEntity<>("", httpHeaders);
        long startGet = System.currentTimeMillis();
        ResponseEntity<byte[]> responseEntityGet = restTemplate.exchange(
                host + String.format(preferencesUri, serverId), HttpMethod.GET, httpEntityGet, byte[].class
        );
        logger.debug("Get HMC[" + host + "]SERVER[" + serverId + "] preferences end, use: " + (System.currentTimeMillis() - startGet) / 1000D + "s");
        DocumentBuilderFactory documentBuilderFactory = DocumentBuilderFactory.newInstance();
        documentBuilderFactory.setExpandEntityReferences(false);
        documentBuilderFactory.setFeature(XMLConstants.FEATURE_SECURE_PROCESSING, true);
        DocumentBuilder documentBuilder = documentBuilderFactory.newDocumentBuilder();
        Document document = documentBuilder.parse(new ByteArrayInputStream(responseEntityGet.getBody()));
        // 查询主机接口返回数据中,feed.entry.id为主机id
        Element entry = (Element) document.getDocumentElement().getElementsByTagName("entry").item(0);
        Element content = (Element) entry.getElementsByTagName("content").item(0);
        Element system = (Element) content.getElementsByTagName("ManagedSystemPcmPreference:ManagedSystemPcmPreference").item(0);
        boolean needToSave = false;
        for (String autoPreference : autoPreferences) {
            if (system.getElementsByTagName(autoPreference) != null && system.getElementsByTagName(autoPreference).getLength() > 0) {
                Element preferenceElement = (Element) system.getElementsByTagName(autoPreference).item(0);
                if (!BooleanUtils.toBoolean(preferenceElement.getTextContent())) {
                    preferenceElement.setTextContent("true");
                    needToSave = true;
                }
            }
        }
        if (needToSave) {
            TransformerFactory transformerFactory = TransformerFactory.newInstance();
            transformerFactory.setFeature(XMLConstants.FEATURE_SECURE_PROCESSING, true);
            Transformer transformer = transformerFactory.newTransformer();
            ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();
            StreamResult streamResult = new StreamResult(byteArrayOutputStream);
            DOMSource domSource = new DOMSource(system);
            transformer.transform(domSource, streamResult);

            HttpEntity<String> httpEntitySet = new HttpEntity<>(byteArrayOutputStream.toString(), httpHeaders);
            long startSet = System.currentTimeMillis();
            ResponseEntity<String> responseEntitySet = restTemplate.exchange(
                    host + String.format(preferencesUri, serverId), HttpMethod.POST, httpEntitySet, String.class
            );
            logger.debug("Set HMC[" + host + "]SERVER[" + serverId + "] preferences end, use: " + (System.currentTimeMillis() - startSet) / 1000D + "s");
            return responseEntitySet.getBody();
        }
        return null;
    }

    private List<HmcPartition> getVioses(String host, String serverId, String sessionId) throws ParserConfigurationException, IOException, SAXException {
        HttpHeaders httpHeaders = new HttpHeaders();
        httpHeaders.set("X-Transaction-ID", sessionId);

        HttpEntity<String> httpEntity = new HttpEntity<>("", httpHeaders);
        long start = System.currentTimeMillis();
        ResponseEntity<byte[]> responseEntity = restTemplate.exchange(
                host + String.format(viosUri, serverId), HttpMethod.GET, httpEntity, byte[].class
        );
        logger.debug("Get HMC[" + host + "]SERVER[" + serverId + "] vios end, use: " + (System.currentTimeMillis() - start) / 1000D + "s");

        List<HmcPartition> vioses = new ArrayList<>();
        DocumentBuilderFactory documentBuilderFactory = DocumentBuilderFactory.newInstance();
        documentBuilderFactory.setExpandEntityReferences(false);
        documentBuilderFactory.setFeature(XMLConstants.FEATURE_SECURE_PROCESSING, true);
        DocumentBuilder documentBuilder = documentBuilderFactory.newDocumentBuilder();
        Document document = documentBuilder.parse(new ByteArrayInputStream(responseEntity.getBody()));
        // 查询主机接口返回数据中,feed.entry.id为主机id
        NodeList entries = document.getDocumentElement().getElementsByTagName("entry");
        for (int i = 0; i < entries.getLength(); i++) {
            Element entry = (Element) entries.item(i);
            String viosId = entry.getElementsByTagName("id").item(0).getTextContent();

            Element content = (Element) entry.getElementsByTagName("content").item(0);
            Element virtualIOServer = (Element) content.getElementsByTagName("VirtualIOServer:VirtualIOServer").item(0);
            Element partitionName = (Element) virtualIOServer.getElementsByTagName("PartitionName").item(0);
            String viosName = partitionName.getTextContent();

            HmcPartition vios = new HmcPartition();
            vios.setUuid(viosId);
            vios.setName(viosName);
            vioses.add(vios);
        }
        return vioses;
    }

    /**
     * 获取LTM监控数据
     *
     * @param vendors 平台列表
     * @param sessionIds  每个HMC平台登录的Session
     * @param servers 每个HMC平台拥有的主机
     * @param vioses 每个主机拥有的vios
     * @return List<GaugeMetricFamily>
     */
    private List<GaugeMetricFamily> longTermMonitor(List<CloudVendor> vendors, Map<String, String> sessionIds, MultiValueMap<String, HmcServer> servers, MultiValueMap<String, HmcPartition> vioses) throws ParserConfigurationException {
        DocumentBuilderFactory documentBuilderFactory = DocumentBuilderFactory.newInstance();
        documentBuilderFactory.setExpandEntityReferences(false);
        documentBuilderFactory.setFeature(XMLConstants.FEATURE_SECURE_PROCESSING, true);
        DocumentBuilder documentBuilder = documentBuilderFactory.newDocumentBuilder();


        GaugeMetricFamily serverCpuTotal = new GaugeMetricFamily(Constants.BOCLOUD_HMC + "ltm_server_cpu_total", "help", labels);
        GaugeMetricFamily serverCpuAvailable = new GaugeMetricFamily(Constants.BOCLOUD_HMC + "ltm_server_cpu_available", "help", labels);
        GaugeMetricFamily serverMemTotal = new GaugeMetricFamily(Constants.BOCLOUD_HMC + "ltm_server_mem_total", "help", labels);
        GaugeMetricFamily serverMemAvailable = new GaugeMetricFamily(Constants.BOCLOUD_HMC + "ltm_server_mem_available", "help", labels);
        GaugeMetricFamily serverHEAReceivePackage = new GaugeMetricFamily(Constants.BOCLOUD_HMC + "ltm_server_hea_rp", "help", labels);
        GaugeMetricFamily serverHEAReceivePackageTotal = new GaugeMetricFamily(Constants.BOCLOUD_HMC + "ltm_server_hea_rp_t", "help", labels);
        GaugeMetricFamily serverHEASendPackage = new GaugeMetricFamily(Constants.BOCLOUD_HMC + "ltm_server_hea_sp", "help", labels);
        GaugeMetricFamily serverHEASendPackageTotal = new GaugeMetricFamily(Constants.BOCLOUD_HMC + "ltm_server_hea_sp_t", "help", labels);
        GaugeMetricFamily serverHEADropPackage = new GaugeMetricFamily(Constants.BOCLOUD_HMC + "ltm_server_hea_dp", "help", labels);
        GaugeMetricFamily serverHEADropPackageTotal = new GaugeMetricFamily(Constants.BOCLOUD_HMC + "ltm_server_hea_dp_t", "help", labels);
        GaugeMetricFamily serverHEAReceiveByte = new GaugeMetricFamily(Constants.BOCLOUD_HMC + "ltm_server_hea_rb", "help", labels);
        GaugeMetricFamily serverHEAReceiveByteTotal = new GaugeMetricFamily(Constants.BOCLOUD_HMC + "ltm_server_hea_rb_t", "help", labels);
        GaugeMetricFamily serverHEASendByte = new GaugeMetricFamily(Constants.BOCLOUD_HMC + "ltm_server_hea_sb", "help", labels);
        GaugeMetricFamily serverHEASendByteTotal = new GaugeMetricFamily(Constants.BOCLOUD_HMC + "ltm_server_hea_sb_t", "help", labels);

        GaugeMetricFamily viosNetworkReceivePackage = new GaugeMetricFamily(Constants.BOCLOUD_HMC + "ltm_vios_network_rp", "help", labels);
        GaugeMetricFamily viosNetworkReceivePackageTotal = new GaugeMetricFamily(Constants.BOCLOUD_HMC + "ltm_vios_network_rp_t", "help", labels);
        GaugeMetricFamily viosNetworkSendPackage = new GaugeMetricFamily(Constants.BOCLOUD_HMC + "ltm_vios_network_sp", "help", labels);
        GaugeMetricFamily viosNetworkSendPackageTotal = new GaugeMetricFamily(Constants.BOCLOUD_HMC + "ltm_vios_network_sp_t", "help", labels);
        GaugeMetricFamily viosNetworkDropPackage = new GaugeMetricFamily(Constants.BOCLOUD_HMC + "ltm_vios_network_dp", "help", labels);
        GaugeMetricFamily viosNetworkDropPackageTotal = new GaugeMetricFamily(Constants.BOCLOUD_HMC + "ltm_vios_network_dp_t", "help", labels);
        GaugeMetricFamily viosNetworkReceiveByte = new GaugeMetricFamily(Constants.BOCLOUD_HMC + "ltm_vios_network_rb", "help", labels);
        GaugeMetricFamily viosNetworkReceiveByteTotal = new GaugeMetricFamily(Constants.BOCLOUD_HMC + "ltm_vios_network_rb_t", "help", labels);
        GaugeMetricFamily viosNetworkSendByte = new GaugeMetricFamily(Constants.BOCLOUD_HMC + "ltm_vios_network_sb", "help", labels);
        GaugeMetricFamily viosNetworkSendByteTotal = new GaugeMetricFamily(Constants.BOCLOUD_HMC + "ltm_vios_network_sb_t", "help", labels);
        GaugeMetricFamily viosStorageWriteNum = new GaugeMetricFamily(Constants.BOCLOUD_HMC + "ltm_vios_storage_wn", "help", labels);
        GaugeMetricFamily viosStorageWriteNumTotal = new GaugeMetricFamily(Constants.BOCLOUD_HMC + "ltm_vios_storage_wn_t", "help", labels);
        GaugeMetricFamily viosStorageReadNum = new GaugeMetricFamily(Constants.BOCLOUD_HMC + "ltm_vios_storage_rn", "help", labels);
        GaugeMetricFamily viosStorageReadNumTotal = new GaugeMetricFamily(Constants.BOCLOUD_HMC + "ltm_vios_storage_rn_t", "help", labels);
        GaugeMetricFamily viosStorageWriteByte = new GaugeMetricFamily(Constants.BOCLOUD_HMC + "ltm_vios_storage_wb", "help", labels);
        GaugeMetricFamily viosStorageWriteByteTotal = new GaugeMetricFamily(Constants.BOCLOUD_HMC + "ltm_vios_storage_wb_t", "help", labels);
        GaugeMetricFamily viosStorageReadByte = new GaugeMetricFamily(Constants.BOCLOUD_HMC + "ltm_vios_storage_rb", "help", labels);
        GaugeMetricFamily viosStorageReadByteTotal = new GaugeMetricFamily(Constants.BOCLOUD_HMC + "ltm_vios_storage_rb_t", "help", labels);

        GaugeMetricFamily partitionCpuMaxCount = new GaugeMetricFamily(Constants.BOCLOUD_HMC + "proc_partition_cpu_max_count", "help", labels);
        GaugeMetricFamily partitionCpuMaxUnit = new GaugeMetricFamily(Constants.BOCLOUD_HMC + "proc_partition_cpu_max_unit", "help", labels);
        GaugeMetricFamily partitionCpuEntitledUnit = new GaugeMetricFamily(Constants.BOCLOUD_HMC + "proc_partition_cpu_entitled_unit", "help", labels);
        GaugeMetricFamily partitionCpuUtilizedCappedUnit = new GaugeMetricFamily(Constants.BOCLOUD_HMC + "proc_partition_cpu_utilized_capped_unit", "help", labels);
        GaugeMetricFamily partitionCpuUtilizedUncappedUnit = new GaugeMetricFamily(Constants.BOCLOUD_HMC + "proc_partition_cpu_utilized_uncapped_unit", "help", labels);
        GaugeMetricFamily partitionCpuIdleUnit = new GaugeMetricFamily(Constants.BOCLOUD_HMC + "proc_partition_cpu_idle_unit", "help", labels);
        GaugeMetricFamily partitionCpuDonatedUnit = new GaugeMetricFamily(Constants.BOCLOUD_HMC + "proc_partition_cpu_donated_unit", "help", labels);
        GaugeMetricFamily partitionMemLogical = new GaugeMetricFamily(Constants.BOCLOUD_HMC + "proc_partition_mem_logical", "help", labels);
        GaugeMetricFamily partitionMemPhysical = new GaugeMetricFamily(Constants.BOCLOUD_HMC + "proc_partition_mem_physical", "help", labels);

        for (CloudVendor vendor : vendors) {
            try {
                String sessionId = sessionIds.get(String.valueOf(vendor.getId()));
                HttpHeaders httpHeaders = new HttpHeaders();
                httpHeaders.set("X-Transaction-ID", sessionId);
                HttpEntity<String> httpEntity = new HttpEntity<>("", httpHeaders);

                // 查询监控数据列表
                Map<HmcServer, Future<byte[]>> serverXmlFutureMap = new HashMap<>();
                for (HmcServer server : servers.get(String.valueOf(vendor.getId()))) {
                    Future<byte[]> future = executorService.submit(() -> {
                        // 获取监控数据，从五分钟之前开始获取
                        String address = JSONObject.parseObject(vendor.getAuthentication()).getString("address");
                        String url = address + String.format(
                                ltmUri, server.getUuid(), new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ssZ").format(DateUtils.addMinutes(new Date(), -5))
                        );
                        logger.debug("Get LTM HMC[" + vendor.getName() + "] entry: ---> " + url);
                        long start = System.currentTimeMillis();
                        ResponseEntity<byte[]> responseEntity = restTemplate.exchange(
                                url, HttpMethod.GET, httpEntity, byte[].class
                        );
                        logger.debug("Get LTM HMC[" + vendor.getName() + "] entry end, use: " + (System.currentTimeMillis() - start) / 1000D + "s");
                        return responseEntity.getBody();
                    });
                    serverXmlFutureMap.put(server, future);
                }

                // 查询监控数据
                Map<HmcServer, Future<String>> serverJSONFutureMap = new HashMap<>();
                MultiValueMap<HmcServer, Future<String>> viosJSONFutureMap = new LinkedMultiValueMap<>();
                for (Map.Entry<HmcServer, Future<byte[]>> futureEntry : serverXmlFutureMap.entrySet()) {
                    if (futureEntry.getValue().get() == null || futureEntry.getValue().get().length == 0) {
                        logger.warn("Get LTM HMC[" + vendor.getName() + "] entry empty! Please check this server and hmc server time/zone!");
                        continue;
                    }
                    Document document = documentBuilder.parse(new ByteArrayInputStream(futureEntry.getValue().get()));
                    // 监控数据中,
                    // feed.entry.id 数据id, 例如: 9354532c-aa2d-4b8d-92c0-97ea5ef77a53
                    // feed.entry.updated 数据时间, 例如: 2019-10-12T15:15:30.000+09:00
                    // feed.entry.link 获取该数据的地址, 例如: "https://192.168.2.69:443/rest/api/pcm/ManagedSystem/87caf195-abef-3567-9766-cc5e326f4995/RawMetrics/LongTermMonitor/LTM_8233-E8B*060713T_phyp_20191012T151530+0900.json
                    // feed.entry.category 数据类型, 例如: phyp、vios_1、vios_2等等
                    NodeList entryNodeList = document.getDocumentElement().getElementsByTagName("entry");
                    logger.debug("Get LTM HMC[" + vendor.getName() + "] entry: <--- " + entryNodeList.getLength());
                    if (entryNodeList.getLength() > 0) {
                        // prometheus每次调用只获取最新时间点的数据，所以这里要比对当前获取结果的最大时间
                        List<Entry> allEntries = new ArrayList<>();
                        for (int i = 0; i < entryNodeList.getLength(); i++) {
                            Element entryNode = (Element) entryNodeList.item(i);
                            // 获取数据id
                            String id = entryNode.getElementsByTagName("id").item(0).getTextContent();
                            // 获取数据类型
                            String category = ((Element) entryNode.getElementsByTagName("category").item(0)).getAttribute("term");
                            // 获取数据地址
                            String href = ((Element) entryNode.getElementsByTagName("link").item(0)).getAttribute("href");
                            // 获取数据时间
                            Element updated = (Element) entryNode.getElementsByTagName("updated").item(0);
                            ZonedDateTime hmcTime = ZonedDateTime.parse(updated.getTextContent(), DateTimeFormatter.ISO_ZONED_DATE_TIME);
                            ZonedDateTime cmpTime = hmcTime.toInstant().atZone(ZoneId.systemDefault());
                            String date = cmpTime.format(DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));

                            allEntries.add(new Entry(id, category, href, date));
                        }
                        logger.debug("Get LTM HMC[" + vendor.getName() + "] entry[All]: <--- " + JSON.toJSONString(allEntries));
                        List<Entry> effectiveEntries = new ArrayList<>();
                        Set<String> dataCategorySet = allEntries.stream().map(Entry::getCategory).collect(Collectors.toSet());
                        for (String category : dataCategorySet) {
                            effectiveEntries.add(allEntries.stream().filter(entry -> entry.getCategory().equals(category)).max(Comparator.comparing(Entry::getDate)).get());
                        }
                        logger.debug("Get LTM HMC[" + vendor.getName() + "] entry[Effective]: <--- " + JSON.toJSONString(effectiveEntries));
                        for (Entry entry : effectiveEntries) {
                            logger.debug("Get LTM HMC[" + vendor.getName() + "] data: ---> " + entry.getHref());
                            long start = System.currentTimeMillis();
                            Future<String> future = executorService.submit(() -> restTemplate.exchange(entry.getHref(), HttpMethod.GET, httpEntity, String.class).getBody());
                            logger.debug("Get LTM HMC[" + vendor.getName() + "] entry end, use: " + (System.currentTimeMillis() - start) / 1000D + "s");
                            if ("phyp".equals(entry.getCategory())) {
                                serverJSONFutureMap.put(futureEntry.getKey(), future);
                            } else {
                                viosJSONFutureMap.add(futureEntry.getKey(), future);
                            }
                        }
                    }
                }

                // 解析主机监控数据
                for (Map.Entry<HmcServer, Future<String>> futureEntry : serverJSONFutureMap.entrySet()) {
                    String serverId = futureEntry.getKey().getUuid();
                    String serverName = futureEntry.getKey().getName();

                    JSONObject jsonObject = JSON.parseObject(futureEntry.getValue().get());
                    JSONObject utilInfo = jsonObject.getJSONObject("systemUtil").getJSONObject("utilInfo");
                    JSONObject utilSample = jsonObject.getJSONObject("systemUtil").getJSONObject("utilSample");
                    // 获取CPU指标
                    List<String> serverCaptions = Arrays.asList(String.valueOf(vendor.getId()), serverId, serverName, "", "");
                    serverCpuTotal.addMetric(serverCaptions, utilSample.getJSONObject("processor").getDoubleValue("totalProcUnits"));
                    serverCpuAvailable.addMetric(serverCaptions, utilSample.getJSONObject("processor").getDoubleValue("availableProcUnits"));
                    logger.debug("Get LTM HMC[" + vendor.getName() + "]SERVER[" + serverName + "]CPU: <--- " + utilSample.getJSONObject("processor").toJSONString());
                    // 获取内存指标
                    serverMemTotal.addMetric(serverCaptions, utilSample.getJSONObject("memory").getDoubleValue("totalMem"));
                    serverMemAvailable.addMetric(serverCaptions, utilSample.getJSONObject("memory").getDoubleValue("availableMem"));
                    logger.debug("Get LTM HMC[" + vendor.getName() + "]SERVER[" + serverName + "]MEM: <--- " + utilSample.getJSONObject("memory").toJSONString());
                    // 获取网卡指标
                    JSONArray heAdapters = utilSample.getJSONObject("network").getJSONArray("heAdapters");
                    if (heAdapters != null) {
                        JSONArray physicalPorts = new JSONArray();
                        heAdapters.forEach(value -> physicalPorts.addAll(((JSONObject) value).getJSONArray("physicalPorts")));
                        for (int j = 0; j < physicalPorts.size(); j++) {
                            JSONObject physicalPort = physicalPorts.getJSONObject(j);
                            String adapterId = physicalPort.getString("id");
                            String adapterLocation = physicalPort.getString("physicalLocation");
                            List<String> heaCaptions = Arrays.asList(String.valueOf(vendor.getId()), adapterId, adapterLocation, serverId, serverName);
                            serverHEAReceivePackage.addMetric(heaCaptions, physicalPort.getDoubleValue("receivedPackets"));
                            serverHEASendPackage.addMetric(heaCaptions, physicalPort.getDoubleValue("sentPackets"));
                            serverHEADropPackage.addMetric(heaCaptions, physicalPort.getDoubleValue("droppedPackets"));
                            serverHEAReceiveByte.addMetric(heaCaptions, MathUtils.round(physicalPort.getDoubleValue("receivedBytes") / 1024, 2));
                            serverHEASendByte.addMetric(heaCaptions, MathUtils.round(physicalPort.getDoubleValue("sentBytes") / 1024, 2));
                            logger.debug("Get LTM HMC[" + vendor.getName() + "]SERVER[" + serverName + "]HEA[" + adapterLocation + ".PORT" + adapterId + "]: <--- " + physicalPort.toJSONString());
                        }
                        serverHEAReceivePackageTotal.addMetric(serverCaptions, physicalPorts.stream().mapToDouble(value -> ((JSONObject) value).getDoubleValue("receivedPackets")).sum());
                        serverHEASendPackageTotal.addMetric(serverCaptions, physicalPorts.stream().mapToDouble(value -> ((JSONObject) value).getDoubleValue("sentPackets")).sum());
                        serverHEADropPackageTotal.addMetric(serverCaptions, physicalPorts.stream().mapToDouble(value -> ((JSONObject) value).getDoubleValue("droppedPackets")).sum());
                        serverHEAReceiveByteTotal.addMetric(serverCaptions, physicalPorts.stream().mapToDouble(value -> ((JSONObject) value).getDoubleValue("receivedBytes")).sum());
                        serverHEASendByteTotal.addMetric(serverCaptions, physicalPorts.stream().mapToDouble(value -> ((JSONObject) value).getDoubleValue("sentBytes")).sum());
                    }
                    // 获取逻辑分区各种指标
                    JSONArray lparsUtil = utilSample.getJSONArray("lparsUtil");
                    if (lparsUtil != null) {
                        for (int i = 0; i < lparsUtil.size(); i++) {
                            JSONObject partition = lparsUtil.getJSONObject(i);
                            String partitionId = partition.getString("uuid");
                            String partitionName = partition.getString("name");
                            List<String> partitionCaption = Arrays.asList(String.valueOf(vendor.getId()), partitionId, partitionName, serverId, serverName);
                            // 获取CPU指标
                            partitionCpuMaxUnit.addMetric(partitionCaption, partition.getJSONObject("processor").getDoubleValue("maxProcUnits"));
                            partitionCpuMaxCount.addMetric(partitionCaption, partition.getJSONObject("processor").getDoubleValue("maxVirtualProcessors"));
                            partitionCpuEntitledUnit.addMetric(partitionCaption, partition.getJSONObject("processor").getDoubleValue("entitledProcCycles"));
                            partitionCpuUtilizedCappedUnit.addMetric(partitionCaption, partition.getJSONObject("processor").getDoubleValue("utilizedCappedProcCycles"));
                            partitionCpuUtilizedUncappedUnit.addMetric(partitionCaption, partition.getJSONObject("processor").getDoubleValue("utilizedUnCappedProcCycles"));
                            partitionCpuIdleUnit.addMetric(partitionCaption, partition.getJSONObject("processor").getDoubleValue("idleProcCycles"));
                            partitionCpuDonatedUnit.addMetric(partitionCaption, partition.getJSONObject("processor").getDoubleValue("donatedProcCycles"));
                            logger.debug("Get LTM HMC[" + vendor.getName() + "]SERVER[" + serverName + "]PARTITION[" + partitionName + "CPU: <--- " + partition.getJSONObject("processor").toJSONString());
                            // 获取内存指标
                            partitionMemLogical.addMetric(partitionCaption, partition.getJSONObject("memory").getDoubleValue("logicalMem"));
                            partitionMemPhysical.addMetric(partitionCaption, partition.getJSONObject("memory").getDoubleValue("backedPhysicalMem"));
                            logger.debug("Get LTM HMC[" + vendor.getName() + "]SERVER[" + serverName + "]PARTITION[" + partitionName + "MEM: <--- " + partition.getJSONObject("memory").toJSONString());
                        }
                    }

                }

                // 解析vios监控数据
                for (Map.Entry<HmcServer, List<Future<String>>> futureEntry : viosJSONFutureMap.entrySet()) {
                    HmcServer server = futureEntry.getKey();
                    String serverId = futureEntry.getKey().getUuid();
                    String serverName = futureEntry.getKey().getName();
                    for (Future<String> futures : futureEntry.getValue()) {
                        JSONObject jsonObject = JSON.parseObject(futures.get());
                        JSONObject utilInfo = jsonObject.getJSONObject("systemUtil").getJSONObject("utilInfo");
                        JSONObject utilSample = jsonObject.getJSONObject("systemUtil").getJSONObject("utilSample");

                        JSONObject vios = utilSample.getJSONArray("viosUtil").getJSONObject(0);
                        String viosName = vios.getString("name");
                        // 这里需要判断viosName是否为空。比如：如果VIOS关机了，该值为空，要尝试打印错误日志
                        if (StringUtils.isEmpty(viosName)) {
                            JSONArray errorInfos = utilSample.getJSONArray("errorInfo");
                            if (errorInfos != null && errorInfos.size() > 0) {
                                errorInfos.forEach(value -> logger.error("Get PROC HMC[" + vendor.getName() + "] data: <--- error: [" + ((JSONObject) value).getString("errId") + "]" + ((JSONObject) value).getString("errMsg")));
                            }
                        } else {
                            String viosId = vioses.get(server.getUuid()).stream().filter(partition -> partition.getName().equals(viosName)).findFirst().get().getUuid();
                            List<String> viosCaptions = Arrays.asList(String.valueOf(vendor.getId()), viosId, viosName, "", "");
                            // 获取网络指标
                            JSONArray allNetworkAdapters = new JSONArray();
                            if (vios.getJSONObject("network").containsKey("genericAdapters")) {
                                allNetworkAdapters.addAll(vios.getJSONObject("network").getJSONArray("genericAdapters"));
                            }
                            if (vios.getJSONObject("network").containsKey("sharedAdapters")) {
                                allNetworkAdapters.addAll(vios.getJSONObject("network").getJSONArray("sharedAdapters"));
                            }
                            for (int i = 0; i < allNetworkAdapters.size(); i++) {
                                JSONObject adapter = allNetworkAdapters.getJSONObject(i);
                                String adapterId = adapter.getString("id");
                                String adapterLocation = adapter.getString("physicalLocation");
                                List<String> viosNetworkCaptions = Arrays.asList(String.valueOf(vendor.getId()), adapterLocation, adapterId, viosId, viosName);
                                viosNetworkReceivePackage.addMetric(viosNetworkCaptions, adapter.getDoubleValue("receivedPackets"));
                                viosNetworkSendPackage.addMetric(viosNetworkCaptions, adapter.getDoubleValue("sentPackets"));
                                viosNetworkDropPackage.addMetric(viosNetworkCaptions, adapter.getDoubleValue("droppedPackets"));
                                viosNetworkReceiveByte.addMetric(viosNetworkCaptions, MathUtils.round(adapter.getDoubleValue("receivedBytes") / 1024, 2));
                                viosNetworkSendByte.addMetric(viosNetworkCaptions, MathUtils.round(adapter.getDoubleValue("sentBytes") / 1024, 2));
                                logger.debug("Get LTM HMC[" + vendor.getName() + "]SERVER[" + serverName + "]VIOS[" + viosName + "]NETWORK[" + adapterId + "]: <--- " + adapter.toJSONString());
                            }
                            viosNetworkReceivePackageTotal.addMetric(viosCaptions, allNetworkAdapters.stream().mapToDouble(value -> ((JSONObject) value).getDoubleValue("receivedPackets")).sum());
                            viosNetworkSendPackageTotal.addMetric(viosCaptions, allNetworkAdapters.stream().mapToDouble(value -> ((JSONObject) value).getDoubleValue("sentPackets")).sum());
                            viosNetworkDropPackageTotal.addMetric(viosCaptions, allNetworkAdapters.stream().mapToDouble(value -> ((JSONObject) value).getDoubleValue("droppedPackets")).sum());
                            viosNetworkReceiveByteTotal.addMetric(viosCaptions, allNetworkAdapters.stream().mapToDouble(value -> ((JSONObject) value).getDoubleValue("receivedBytes")).sum());
                            viosNetworkSendByteTotal.addMetric(viosCaptions, allNetworkAdapters.stream().mapToDouble(value -> ((JSONObject) value).getDoubleValue("sentBytes")).sum());
                            // 获取存储指标
                            JSONArray allStroageAdapters = new JSONArray();
                            if (vios.getJSONObject("storage").containsKey("fiberChannelAdapters")) {
                                allStroageAdapters.addAll(vios.getJSONObject("storage").getJSONArray("fiberChannelAdapters"));
                            }
                            if (vios.getJSONObject("storage").containsKey("genericPhysicalAdapters")) {
                                allStroageAdapters.addAll(vios.getJSONObject("storage").getJSONArray("genericPhysicalAdapters"));
                            }
                            if (vios.getJSONObject("storage").containsKey("genericVirtualAdapters")) {
                                allStroageAdapters.addAll(vios.getJSONObject("storage").getJSONArray("genericVirtualAdapters"));
                            }
                            if (vios.getJSONObject("storage").containsKey("sharedStoragePools")) {
                                allStroageAdapters.addAll(vios.getJSONObject("storage").getJSONArray("sharedStoragePools"));
                            }
                            for (int i = 0; i < allStroageAdapters.size(); i++) {
                                JSONObject adapter = allStroageAdapters.getJSONObject(i);
                                String adapterId = adapter.getString("id");
                                String adapterLocation = adapter.getString("physicalLocation");
                                List<String> viosStorageCaptions = Arrays.asList(String.valueOf(vendor.getId()), adapterLocation, adapterId, viosId, viosName);
                                viosStorageReadByte.addMetric(viosStorageCaptions, MathUtils.round(adapter.getDoubleValue("readBytes") / 1024, 2));
                                viosStorageWriteByte.addMetric(viosStorageCaptions,  MathUtils.round(adapter.getDoubleValue("writeBytes") / 1024, 2));
                                viosStorageReadNum.addMetric(viosStorageCaptions, adapter.getDoubleValue("numOfReads"));
                                viosStorageWriteNum.addMetric(viosStorageCaptions, adapter.getDoubleValue("numOfWrites"));
                                logger.debug("Get LTM HMC[" + vendor.getName() + "]SERVER[" + serverName + "]VIOS[" + viosName + "]STORAGE[" + adapterId + "]: <--- " + adapter.toJSONString());
                            }
                            viosStorageReadByteTotal.addMetric(viosCaptions, allStroageAdapters.stream().mapToDouble(value -> ((JSONObject) value).getDoubleValue("readBytes")).sum());
                            viosStorageReadNumTotal.addMetric(viosCaptions, allStroageAdapters.stream().mapToDouble(value -> ((JSONObject) value).getDoubleValue("numOfReads")).sum());
                            viosStorageWriteByteTotal.addMetric(viosCaptions, allStroageAdapters.stream().mapToDouble(value -> ((JSONObject) value).getDoubleValue("writeBytes")).sum());
                            viosStorageWriteNumTotal.addMetric(viosCaptions, allStroageAdapters.stream().mapToDouble(value -> ((JSONObject) value).getDoubleValue("numOfWrites")).sum());
                        }
                    }
                }
            } catch (SAXException | IOException | ExecutionException e) {
                logger.error("Get LTM HMC[" + vendor.getName() + "] LongTermMonitor error:{} ", e.getMessage(), e);
            } catch (InterruptedException e) {
                Thread.currentThread().interrupt();
                logger.error("Interrupted LTM HMC[" + vendor.getName() + "] LongTermMonitor");
                break;
            }
        }
        return Arrays.asList(
                serverCpuTotal, serverCpuAvailable, serverMemTotal, serverMemAvailable,
                serverHEAReceivePackage, serverHEASendPackage, serverHEADropPackage, serverHEAReceiveByte, serverHEASendByte,
                serverHEAReceivePackageTotal, serverHEASendPackageTotal, serverHEADropPackageTotal, serverHEAReceiveByteTotal, serverHEASendByteTotal,
                viosNetworkReceivePackage, viosNetworkSendPackage, viosNetworkDropPackage, viosNetworkReceiveByte, viosNetworkSendByte,
                viosNetworkReceivePackageTotal, viosNetworkSendPackageTotal, viosNetworkDropPackageTotal, viosNetworkReceiveByteTotal, viosNetworkSendByteTotal,
                viosStorageReadByte, viosStorageWriteByte, viosStorageReadNum, viosStorageWriteNum,
                viosStorageReadByteTotal, viosStorageWriteByteTotal, viosStorageReadNumTotal, viosStorageWriteNumTotal
        );
    }

    /**
     * 获取实时监控数据
     * @param vendors 平台列表
     * @param sessionIds  每个HMC平台登录的Session
     * @param servers 每个HMC平台拥有的主机
     * @param vioses 每个主机拥有的vios
     * @return List<GaugeMetricFamily>
     */
    private List<GaugeMetricFamily> processorMonitor(List<CloudVendor> vendors, Map<String, String> sessionIds, MultiValueMap<String, HmcServer> servers, MultiValueMap<String, HmcPartition> vioses) throws ParserConfigurationException {
        DocumentBuilderFactory documentBuilderFactory = DocumentBuilderFactory.newInstance();
        documentBuilderFactory.setExpandEntityReferences(false);
        documentBuilderFactory.setFeature(XMLConstants.FEATURE_SECURE_PROCESSING, true);
        DocumentBuilder documentBuilder = documentBuilderFactory.newDocumentBuilder();


        GaugeMetricFamily serverCpuTotal = new GaugeMetricFamily(Constants.BOCLOUD_HMC + "proc_server_cpu_total", "help", labels);
        GaugeMetricFamily serverCpuAvailable = new GaugeMetricFamily(Constants.BOCLOUD_HMC + "proc_server_cpu_available", "help", labels);
        GaugeMetricFamily serverCpuUsed = new GaugeMetricFamily(Constants.BOCLOUD_HMC + "proc_server_cpu_used", "help", labels);
        GaugeMetricFamily serverCpuUtilized = new GaugeMetricFamily(Constants.BOCLOUD_HMC + "proc_server_cpu_utilized", "help", labels);
        GaugeMetricFamily serverMemTotal = new GaugeMetricFamily(Constants.BOCLOUD_HMC + "proc_server_mem_total", "help", labels);
        GaugeMetricFamily serverMemAvailable = new GaugeMetricFamily(Constants.BOCLOUD_HMC + "proc_server_mem_available", "help", labels);
        GaugeMetricFamily serverMemUsed = new GaugeMetricFamily(Constants.BOCLOUD_HMC + "proc_server_mem_used", "help", labels);
        GaugeMetricFamily serverMemPartition = new GaugeMetricFamily(Constants.BOCLOUD_HMC + "proc_server_mem_partition", "help", labels);
        GaugeMetricFamily serverMemSystem = new GaugeMetricFamily(Constants.BOCLOUD_HMC + "proc_server_mem_system", "help", labels);
        GaugeMetricFamily serverShareCpuPoolAssigned = new GaugeMetricFamily(Constants.BOCLOUD_HMC + "proc_server_share_cpu_pool_assigned", "help", labels);
        GaugeMetricFamily serverShareCpuPoolConfigured = new GaugeMetricFamily(Constants.BOCLOUD_HMC + "proc_server_share_cpu_pool_configured", "help", labels);
        GaugeMetricFamily serverShareCpuPoolAvailable = new GaugeMetricFamily(Constants.BOCLOUD_HMC + "proc_server_share_cpu_pool_available", "help", labels);
        GaugeMetricFamily serverShareCpuPoolBorrowed = new GaugeMetricFamily(Constants.BOCLOUD_HMC + "proc_server_share_cpu_pool_borrowed", "help", labels);
        GaugeMetricFamily serverShareCpuPoolUtilized = new GaugeMetricFamily(Constants.BOCLOUD_HMC + "proc_server_share_cpu_pool_utilized", "help", labels);
        GaugeMetricFamily serverHEAReceivePackage = new GaugeMetricFamily(Constants.BOCLOUD_HMC + "proc_server_hea_rp", "help", labels);
        GaugeMetricFamily serverHEAReceivePackageTotal = new GaugeMetricFamily(Constants.BOCLOUD_HMC + "proc_server_hea_rp_t", "help", labels);
        GaugeMetricFamily serverHEASendPackage = new GaugeMetricFamily(Constants.BOCLOUD_HMC + "proc_server_hea_sp", "help", labels);
        GaugeMetricFamily serverHEASendPackageTotal = new GaugeMetricFamily(Constants.BOCLOUD_HMC + "proc_server_hea_sp_t", "help", labels);
        GaugeMetricFamily serverHEADropPackage = new GaugeMetricFamily(Constants.BOCLOUD_HMC + "proc_server_hea_dp", "help", labels);
        GaugeMetricFamily serverHEADropPackageTotal = new GaugeMetricFamily(Constants.BOCLOUD_HMC + "proc_server_hea_dp_t", "help", labels);
        GaugeMetricFamily serverHEAReceiveByte = new GaugeMetricFamily(Constants.BOCLOUD_HMC + "proc_server_hea_rb", "help", labels);
        GaugeMetricFamily serverHEAReceiveByteTotal = new GaugeMetricFamily(Constants.BOCLOUD_HMC + "proc_server_hea_rb_t", "help", labels);
        GaugeMetricFamily serverHEASendByte = new GaugeMetricFamily(Constants.BOCLOUD_HMC + "proc_server_hea_sb", "help", labels);
        GaugeMetricFamily serverHEASendByteTotal = new GaugeMetricFamily(Constants.BOCLOUD_HMC + "proc_server_hea_sb_t", "help", labels);
        GaugeMetricFamily serverHEATransferByte = new GaugeMetricFamily(Constants.BOCLOUD_HMC + "proc_server_hea_tb", "help", labels);
        GaugeMetricFamily serverHEATransferByteTotal = new GaugeMetricFamily(Constants.BOCLOUD_HMC + "proc_server_hea_tb_t", "help", labels);

        GaugeMetricFamily viosCpuMaxCount = new GaugeMetricFamily(Constants.BOCLOUD_HMC + "proc_vios_cpu_max_count", "help", labels);
        GaugeMetricFamily viosCpuMaxUnit = new GaugeMetricFamily(Constants.BOCLOUD_HMC + "proc_vios_cpu_max_unit", "help", labels);
        GaugeMetricFamily viosCpuEntitledUnit = new GaugeMetricFamily(Constants.BOCLOUD_HMC + "proc_vios_cpu_entitled_unit", "help", labels);
        GaugeMetricFamily viosCpuUtilizedUnit = new GaugeMetricFamily(Constants.BOCLOUD_HMC + "proc_vios_cpu_utilized_unit", "help", labels);
        GaugeMetricFamily viosCpuUtilizedCappedUnit = new GaugeMetricFamily(Constants.BOCLOUD_HMC + "proc_vios_cpu_utilized_capped_unit", "help", labels);
        GaugeMetricFamily viosCpuUtilizedUncappedUnit = new GaugeMetricFamily(Constants.BOCLOUD_HMC + "proc_vios_cpu_utilized_uncapped_unit", "help", labels);
        GaugeMetricFamily viosCpuIdleUnit = new GaugeMetricFamily(Constants.BOCLOUD_HMC + "proc_vios_cpu_idle_unit", "help", labels);
        GaugeMetricFamily viosCpuDonatedUnit = new GaugeMetricFamily(Constants.BOCLOUD_HMC + "proc_vios_cpu_donated_unit", "help", labels);
        GaugeMetricFamily viosMemTotal = new GaugeMetricFamily(Constants.BOCLOUD_HMC + "proc_vios_mem_total", "help", labels);
        GaugeMetricFamily viosMemUtilized = new GaugeMetricFamily(Constants.BOCLOUD_HMC + "proc_vios_mem_utilized", "help", labels);
        GaugeMetricFamily viosNetworkReceivePackage = new GaugeMetricFamily(Constants.BOCLOUD_HMC + "proc_vios_network_rp", "help", labels);
        GaugeMetricFamily viosNetworkReceivePackageTotal = new GaugeMetricFamily(Constants.BOCLOUD_HMC + "proc_vios_network_rp_t", "help", labels);
        GaugeMetricFamily viosNetworkSendPackage = new GaugeMetricFamily(Constants.BOCLOUD_HMC + "proc_vios_network_sp", "help", labels);
        GaugeMetricFamily viosNetworkSendPackageTotal = new GaugeMetricFamily(Constants.BOCLOUD_HMC + "proc_vios_network_sp_t", "help", labels);
        GaugeMetricFamily viosNetworkDropPackage = new GaugeMetricFamily(Constants.BOCLOUD_HMC + "proc_vios_network_dp", "help", labels);
        GaugeMetricFamily viosNetworkDropPackageTotal = new GaugeMetricFamily(Constants.BOCLOUD_HMC + "proc_vios_network_dp_t", "help", labels);
        GaugeMetricFamily viosNetworkReceiveByte = new GaugeMetricFamily(Constants.BOCLOUD_HMC + "proc_vios_network_rb", "help", labels);
        GaugeMetricFamily viosNetworkReceiveByteTotal = new GaugeMetricFamily(Constants.BOCLOUD_HMC + "proc_vios_network_rb_t", "help", labels);
        GaugeMetricFamily viosNetworkSendByte = new GaugeMetricFamily(Constants.BOCLOUD_HMC + "proc_vios_network_sb", "help", labels);
        GaugeMetricFamily viosNetworkSendByteTotal = new GaugeMetricFamily(Constants.BOCLOUD_HMC + "proc_vios_network_sb_t", "help", labels);
        GaugeMetricFamily viosNetworkTransferByte = new GaugeMetricFamily(Constants.BOCLOUD_HMC + "proc_vios_network_tb", "help", labels);
        GaugeMetricFamily viosNetworkTransferByteTotal = new GaugeMetricFamily(Constants.BOCLOUD_HMC + "proc_vios_network_tb_t", "help", labels);
        GaugeMetricFamily viosStorageWriteNum = new GaugeMetricFamily(Constants.BOCLOUD_HMC + "proc_vios_storage_wn", "help", labels);
        GaugeMetricFamily viosStorageWriteNumTotal = new GaugeMetricFamily(Constants.BOCLOUD_HMC + "proc_vios_storage_wn_t", "help", labels);
        GaugeMetricFamily viosStorageReadNum = new GaugeMetricFamily(Constants.BOCLOUD_HMC + "proc_vios_storage_rn", "help", labels);
        GaugeMetricFamily viosStorageReadNumTotal = new GaugeMetricFamily(Constants.BOCLOUD_HMC + "proc_vios_storage_rn_t", "help", labels);
        GaugeMetricFamily viosStorageWriteByte = new GaugeMetricFamily(Constants.BOCLOUD_HMC + "proc_vios_storage_wb", "help", labels);
        GaugeMetricFamily viosStorageWriteByteTotal = new GaugeMetricFamily(Constants.BOCLOUD_HMC + "proc_vios_storage_wb_t", "help", labels);
        GaugeMetricFamily viosStorageReadByte = new GaugeMetricFamily(Constants.BOCLOUD_HMC + "proc_vios_storage_rb", "help", labels);
        GaugeMetricFamily viosStorageReadByteTotal = new GaugeMetricFamily(Constants.BOCLOUD_HMC + "proc_vios_storage_rb_t", "help", labels);
        GaugeMetricFamily viosStorageTransmitByte = new GaugeMetricFamily(Constants.BOCLOUD_HMC + "proc_vios_storage_tb", "help", labels);
        GaugeMetricFamily viosStorageTransmitByteTotal = new GaugeMetricFamily(Constants.BOCLOUD_HMC + "proc_vios_storage_tb_t", "help", labels);

        GaugeMetricFamily partitionCpuMaxCount = new GaugeMetricFamily(Constants.BOCLOUD_HMC + "proc_partition_cpu_max_count", "help", labels);
        GaugeMetricFamily partitionCpuMaxUnit = new GaugeMetricFamily(Constants.BOCLOUD_HMC + "proc_partition_cpu_max_unit", "help", labels);
        GaugeMetricFamily partitionCpuEntitledUnit = new GaugeMetricFamily(Constants.BOCLOUD_HMC + "proc_partition_cpu_entitled_unit", "help", labels);
        GaugeMetricFamily partitionCpuUtilizedUnit = new GaugeMetricFamily(Constants.BOCLOUD_HMC + "proc_partition_cpu_utilized_unit", "help", labels);
        GaugeMetricFamily partitionCpuUtilizedCappedUnit = new GaugeMetricFamily(Constants.BOCLOUD_HMC + "proc_partition_cpu_utilized_capped_unit", "help", labels);
        GaugeMetricFamily partitionCpuUtilizedUncappedUnit = new GaugeMetricFamily(Constants.BOCLOUD_HMC + "proc_partition_cpu_utilized_uncapped_unit", "help", labels);
        GaugeMetricFamily partitionCpuIdleUnit = new GaugeMetricFamily(Constants.BOCLOUD_HMC + "proc_partition_cpu_idle_unit", "help", labels);
        GaugeMetricFamily partitionCpuDonatedUnit = new GaugeMetricFamily(Constants.BOCLOUD_HMC + "proc_partition_cpu_donated_unit", "help", labels);
        GaugeMetricFamily partitionMemLogical = new GaugeMetricFamily(Constants.BOCLOUD_HMC + "proc_partition_mem_logical", "help", labels);
        GaugeMetricFamily partitionMemPhysical = new GaugeMetricFamily(Constants.BOCLOUD_HMC + "proc_partition_mem_physical", "help", labels);
        GaugeMetricFamily partitionStorageWriteNum = new GaugeMetricFamily(Constants.BOCLOUD_HMC + "proc_partition_storage_wn", "help", labels);
        GaugeMetricFamily partitionStorageWriteNumTotal = new GaugeMetricFamily(Constants.BOCLOUD_HMC + "proc_partition_storage_wn_t", "help", labels);
        GaugeMetricFamily partitionStorageReadNum = new GaugeMetricFamily(Constants.BOCLOUD_HMC + "proc_partition_storage_rn", "help", labels);
        GaugeMetricFamily partitionStorageReadNumTotal = new GaugeMetricFamily(Constants.BOCLOUD_HMC + "proc_partition_storage_rn_t", "help", labels);
        GaugeMetricFamily partitionStorageWriteByte = new GaugeMetricFamily(Constants.BOCLOUD_HMC + "proc_partition_storage_wb", "help", labels);
        GaugeMetricFamily partitionStorageWriteByteTotal = new GaugeMetricFamily(Constants.BOCLOUD_HMC + "proc_partition_storage_wb_t", "help", labels);
        GaugeMetricFamily partitionStorageReadByte = new GaugeMetricFamily(Constants.BOCLOUD_HMC + "proc_partition_storage_rb", "help", labels);
        GaugeMetricFamily partitionStorageReadByteTotal = new GaugeMetricFamily(Constants.BOCLOUD_HMC + "proc_partition_storage_rb_t", "help", labels);
        GaugeMetricFamily partitionStorageTransmitByte = new GaugeMetricFamily(Constants.BOCLOUD_HMC + "proc_partition_storage_tb", "help", labels);
        GaugeMetricFamily partitionStorageTransmitByteTotal = new GaugeMetricFamily(Constants.BOCLOUD_HMC + "proc_partition_storage_tb_t", "help", labels);

        for (CloudVendor vendor : vendors) {
            try {
                String sessionId = sessionIds.get(String.valueOf(vendor.getId()));
                HttpHeaders httpHeaders = new HttpHeaders();
                httpHeaders.set("X-Transaction-ID", sessionId);
                HttpEntity<String> httpEntity = new HttpEntity<>("", httpHeaders);

                // 查询主机监控数据列表
                Map<HmcServer, Future<byte[]>> serverXmlFutureMap = new HashMap<>();
                for (HmcServer server : servers.get(String.valueOf(vendor.getId()))) {
                    Future<byte[]> future = executorService.submit(() -> {
                        // 获取监控数据，从五分钟之前开始获取
                        String address = JSONObject.parseObject(vendor.getAuthentication()).getString("address");
                        String url = address + String.format(
                            procUri, server.getUuid(), new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ssZ").format(DateUtils.addMinutes(new Date(), -5))
                        );
                        logger.debug("Get PROC HMC[" + vendor.getName() + "] entry: ---> " + url);
                        long start = System.currentTimeMillis();
                        ResponseEntity<byte[]> responseEntity = restTemplate.exchange(url, HttpMethod.GET, httpEntity, byte[].class);
                        logger.debug("Get PROC HMC[" + vendor.getName() + "] entry end, use: " + (System.currentTimeMillis() - start) / 1000D + "s");
                        return responseEntity.getBody();
                    });
                    serverXmlFutureMap.put(server, future);
                }

                // 查询主机监控数据和分区监控数据列表
                Map<HmcServer, Future<String>> serverJSONFutureMap = new HashMap<>();
                MultiValueMap<HmcServer, Future<byte[]>> partitionXmpFutureMap = new LinkedMultiValueMap<>();
                for (Map.Entry<HmcServer, Future<byte[]>> futureEntry : serverXmlFutureMap.entrySet()) {
                    if (futureEntry.getValue().get() == null || futureEntry.getValue().get().length == 0) {
                        logger.warn("Get LTM HMC[" + vendor.getName() + "] entry empty! Please check this server and hmc server time/zone!");
                        continue;
                    }
                    Document document = documentBuilder.parse(new ByteArrayInputStream(futureEntry.getValue().get()));
                    // 监控数据中,
                    // feed.entry.id 数据id, 例如: 9354532c-aa2d-4b8d-92c0-97ea5ef77a53
                    // feed.entry.updated 数据时间, 例如: 2019-10-12T15:15:30.000+09:00
                    // feed.entry.link 获取该数据的地址, 例如: "https://192.168.2.69:443/rest/api/pcm/ManagedSystem/87caf195-abef-3567-9766-cc5e326f4995/RawMetrics/LongTermMonitor/LTM_8233-E8B*060713T_phyp_20191012T151530+0900.json
                    // feed.entry.category 数据类型, 例如: ManagedSystem、LogicalPartition
                    NodeList entryNodeList = document.getDocumentElement().getElementsByTagName("entry");
                    logger.debug("Get PROC HMC[" + vendor.getName() + "] entry: <--- " + entryNodeList.getLength());
                    if (entryNodeList.getLength() > 0) {
                        List<Entry> entries = new ArrayList<>();
                        for (int i = 0; i < entryNodeList.getLength(); i++) {
                            Element entryNode = (Element) entryNodeList.item(i);
                            // 获取数据id
                            String id = entryNode.getElementsByTagName("id").item(0).getTextContent();
                            // 获取数据类型
                            String category = ((Element) entryNode.getElementsByTagName("category").item(0)).getAttribute("term");
                            // 获取数据地址
                            String href = ((Element) entryNode.getElementsByTagName("link").item(0)).getAttribute("href");
                            // 获取数据时间
                            Element updated = (Element) entryNode.getElementsByTagName("updated").item(0);
                            ZonedDateTime hmcTime = ZonedDateTime.parse(updated.getTextContent(), DateTimeFormatter.ISO_ZONED_DATE_TIME);
                            ZonedDateTime cmpTime = hmcTime.toInstant().atZone(ZoneId.systemDefault());
                            String date = cmpTime.format(DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));

                            entries.add(new Entry(id, category, href, date));
                        }
                        logger.debug("Get PROC HMC[" + vendor.getName() + "] entry: <--- " + JSON.toJSONString(entries));
                        for (Entry entry : entries) {
                            if ("ManagedSystem".equals(entry.getCategory())) {
                                Future<String> future = executorService.submit(() -> {
                                    logger.debug("Get PROC HMC[" + vendor.getName() + "] data: ---> " + entry.getHref());
                                    long start = System.currentTimeMillis();
                                    ResponseEntity<String> responseEntity = restTemplate.exchange(entry.getHref(), HttpMethod.GET, httpEntity, String.class);
                                    logger.debug("Get PROC HMC[" + vendor.getName() + "] entry end, use: " + (System.currentTimeMillis() - start) / 1000D + "s");
                                    return responseEntity.getBody();
                                });
                                serverJSONFutureMap.put(futureEntry.getKey(), future);
                            } else {
                                Future<byte[]> future = executorService.submit(() -> {
                                    logger.debug("Get PROC HMC[" + vendor.getName() + "]PARTITION entry: ---> " + entry.getHref());
                                    long start = System.currentTimeMillis();
                                    ResponseEntity<byte[]> responseEntity = restTemplate.exchange(entry.getHref(), HttpMethod.GET, httpEntity, byte[].class);
                                    logger.debug("Get PROC HMC[" + vendor.getName() + "]PARTITION entry end, use: " + (System.currentTimeMillis() - start) / 1000D + "s");
                                    return responseEntity.getBody();
                                });
                                partitionXmpFutureMap.add(futureEntry.getKey(), future);
                            }
                        }
                    }
                }

                // 查询分区监控数据
                MultiValueMap<HmcServer, Future<String>> partitionJSONFutureMap = new LinkedMultiValueMap<>();
                for (Map.Entry<HmcServer, List<Future<byte[]>>> futureListEntry : partitionXmpFutureMap.entrySet()) {
                    for (Future<byte[]> futureEntry : futureListEntry.getValue()) {
                        Document partitionDocument = documentBuilder.parse(new ByteArrayInputStream(futureEntry.get()));
                        // 监控数据中,
                        // feed.entry.id 数据id, 例如: 9354532c-aa2d-4b8d-92c0-97ea5ef77a53
                        // feed.entry.updated 数据时间, 例如: 2019-10-12T15:15:30.000+09:00
                        // feed.entry.link 获取该数据的地址, 例如: "https://192.168.2.69:443/rest/api/pcm/ManagedSystem/87caf195-abef-3567-9766-cc5e326f4995/RawMetrics/LongTermMonitor/LTM_8233-E8B*060713T_phyp_20191012T151530+0900.json
                        // feed.entry.category 数据类型, 例如: LogicalPartition
                        NodeList partitionEntryNodeList = partitionDocument.getDocumentElement().getElementsByTagName("entry");
                        logger.debug("Get PROC HMC[" + vendor.getName() + "]PARTITION entry: <--- " + partitionEntryNodeList.getLength());
                        if (partitionEntryNodeList.getLength() > 0) {
                            List<Entry> partitionEntries = new ArrayList<>();
                            for (int i = 0; i < partitionEntryNodeList.getLength(); i++) {
                                Element entryNode = (Element) partitionEntryNodeList.item(i);
                                // 获取数据id
                                String id = entryNode.getElementsByTagName("id").item(0).getTextContent();
                                // 获取数据类型
                                String category = ((Element) entryNode.getElementsByTagName("category").item(0)).getAttribute("term");
                                // 获取数据地址
                                String href = ((Element) entryNode.getElementsByTagName("link").item(0)).getAttribute("href");
                                // 获取数据时间
                                Element updated = (Element) entryNode.getElementsByTagName("updated").item(0);
                                ZonedDateTime hmcTime = ZonedDateTime.parse(updated.getTextContent(), DateTimeFormatter.ISO_ZONED_DATE_TIME);
                                ZonedDateTime cmpTime = hmcTime.toInstant().atZone(ZoneId.systemDefault());
                                String date = cmpTime.format(DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));

                                partitionEntries.add(new Entry(id, category, href, date));
                            }
                            logger.debug("Get PROC HMC[" + vendor.getName() + "]PARTITION entry: <--- " + JSON.toJSONString(partitionEntries));
                            for (Entry partitionEntry : partitionEntries) {
                                Future<String> future = executorService.submit(() -> {
                                    logger.debug("Get PROC HMC[" + vendor.getName() + "]PARTITION data: ---> " + partitionEntry.getHref());
                                    long start = System.currentTimeMillis();
                                    ResponseEntity<String> responseEntity = restTemplate.exchange(partitionEntry.getHref(), HttpMethod.GET, httpEntity, String.class);
                                    logger.debug("Get PROC HMC[" + vendor.getName() + "]PARTITION data end, use: " + (System.currentTimeMillis() - start) / 1000D + "s");
                                    return responseEntity.getBody();
                                });
                                partitionJSONFutureMap.add(futureListEntry.getKey(), future);
                            }
                        }
                    }
                }

                // 解析主机监控数据
                for (Map.Entry<HmcServer, Future<String>> futureEntry : serverJSONFutureMap.entrySet()) {
                    HmcServer server = futureEntry.getKey();
                    String serverId = server.getUuid();
                    String serverName = server.getName();

                    JSONObject jsonObject = JSON.parseObject(futureEntry.getValue().get());
                    JSONObject utilInfo = jsonObject.getJSONObject("systemUtil").getJSONObject("utilInfo");
                    JSONArray utilSamples = jsonObject.getJSONObject("systemUtil").getJSONArray("utilSamples");
                    // prometheus每次调用只获取最新时间点的数据，所以这里要比对当前获取结果的最大时间
                    JSONObject utilSample = null;
                    for (int i = 0; i < utilSamples.size(); i++) {
                        if (utilSample == null || utilSample.getJSONObject("sampleInfo").getString("timeStamp").compareTo(
                            utilSamples.getJSONObject(i).getJSONObject("sampleInfo").getString("timeStamp")
                        ) < 0) {
                            utilSample = utilSamples.getJSONObject(i);
                        }
                    }
                    // 开始处理
                    List<String> serverCaptions = Arrays.asList(String.valueOf(vendor.getId()), serverId, serverName, "", "");
                    // 处理器监控指标
                    serverCpuTotal.addMetric(serverCaptions, utilSample.getJSONObject("serverUtil").getJSONObject("processor").getJSONArray("totalProcUnits").getDoubleValue(0));
                    serverCpuAvailable.addMetric(serverCaptions, utilSample.getJSONObject("serverUtil").getJSONObject("processor").getJSONArray("availableProcUnits").getDoubleValue(0));
                    serverCpuUsed.addMetric(serverCaptions,
                        utilSample.getJSONObject("serverUtil").getJSONObject("processor").getJSONArray("totalProcUnits").getDoubleValue(0) -
                            utilSample.getJSONObject("serverUtil").getJSONObject("processor").getJSONArray("availableProcUnits").getDoubleValue(0)
                    );
                    serverCpuUtilized.addMetric(serverCaptions, utilSample.getJSONObject("serverUtil").getJSONObject("processor").getJSONArray("utilizedProcUnits").getDoubleValue(0));
                    logger.debug("Get PROC HMC[" + vendor.getName() + "]SERVER[" + serverName + "]CPU: <--- " + utilSample.getJSONObject("serverUtil").getJSONObject("processor").toJSONString());
                    // 内存监控指标
                    serverMemTotal.addMetric(serverCaptions, utilSample.getJSONObject("serverUtil").getJSONObject("memory").getJSONArray("totalMem").getDoubleValue(0));
                    serverMemAvailable.addMetric(serverCaptions, utilSample.getJSONObject("serverUtil").getJSONObject("memory").getJSONArray("availableMem").getDoubleValue(0));
                    serverMemUsed.addMetric(serverCaptions,
                        utilSample.getJSONObject("serverUtil").getJSONObject("memory").getJSONArray("totalMem").getDoubleValue(0) -
                            utilSample.getJSONObject("serverUtil").getJSONObject("memory").getJSONArray("availableMem").getDoubleValue(0)
                    );
                    serverMemPartition.addMetric(serverCaptions, utilSample.getJSONObject("serverUtil").getJSONObject("memory").getJSONArray("assignedMemToLpars").getDoubleValue(0));
                    serverMemSystem.addMetric(serverCaptions, utilSample.getJSONObject("systemFirmwareUtil").getJSONArray("assignedMem").getDoubleValue(0));
                    logger.debug("Get PROC HMC[" + vendor.getName() + "]SERVER[" + serverName + "]MEM: <--- " + utilSample.getJSONObject("serverUtil").getJSONObject("memory").toJSONString());
                    // 共享处理器池监控指标
                    JSONArray sharedProcessorPools = utilSample.getJSONObject("serverUtil").getJSONArray("sharedProcessorPool");
                    if (sharedProcessorPools != null) {
                        for (int i = 0; i < sharedProcessorPools.size(); i++) {
                            JSONObject sharedProcessorPool = sharedProcessorPools.getJSONObject(i);
                            List<String> shareProcessorPoolCaption = Arrays.asList(String.valueOf(vendor.getId()), sharedProcessorPool.getString("id"), sharedProcessorPool.getString("name"), serverId, serverName);
                            serverShareCpuPoolAssigned.addMetric(shareProcessorPoolCaption, sharedProcessorPool.getJSONArray("assignedProcUnits").getDoubleValue(0));
                            serverShareCpuPoolAvailable.addMetric(shareProcessorPoolCaption, sharedProcessorPool.getJSONArray("availableProcUnits").getDoubleValue(0));
                            serverShareCpuPoolConfigured.addMetric(shareProcessorPoolCaption,
                                sharedProcessorPool.containsKey("configuredProcUnits") && sharedProcessorPool.getJSONArray("configuredProcUnits").size() > 0 ?
                                    sharedProcessorPool.getJSONArray("configuredProcUnits").getDoubleValue(0) : 0
                            );
                            serverShareCpuPoolBorrowed.addMetric(shareProcessorPoolCaption,
                                sharedProcessorPool.containsKey("borrowedProcUnits") && sharedProcessorPool.getJSONArray("borrowedProcUnits").size() > 0 ?
                                    sharedProcessorPool.getJSONArray("borrowedProcUnits").getDoubleValue(0) : 0
                            );
                            serverShareCpuPoolUtilized.addMetric(shareProcessorPoolCaption, sharedProcessorPool.getJSONArray("utilizedProcUnits").getDoubleValue(0));
                            logger.debug("Get PROC HMC[" + vendor.getName() + "]SERVER[" + serverName + "]SHARE_PROCESSOR_POOL: <--- " + sharedProcessorPool.toJSONString());
                        }
                    }
                    // 物理网卡监控指标
                    JSONArray heAdapters = utilSample.getJSONObject("serverUtil").getJSONObject("network").getJSONArray("headapters");
                    if (heAdapters != null) {
                        JSONArray physicalPorts = new JSONArray();
                        heAdapters.forEach(value -> physicalPorts.addAll(((JSONObject) value).getJSONArray("physicalPorts")));
                        for (int j = 0; j < physicalPorts.size(); j++) {
                            JSONObject physicalPort = physicalPorts.getJSONObject(j);
                            String adapterId = physicalPort.getString("id");
                            String adapterLocation = physicalPort.getString("physicalLocation");
                            List<String> heaCaptions = Arrays.asList(String.valueOf(vendor.getId()), adapterId, adapterLocation, serverId, serverName);
                            serverHEAReceivePackage.addMetric(heaCaptions, physicalPort.getJSONArray("receivedPackets").getDoubleValue(0));
                            serverHEASendPackage.addMetric(heaCaptions, physicalPort.getJSONArray("sentPackets").getDoubleValue(0));
                            serverHEADropPackage.addMetric(heaCaptions, physicalPort.getJSONArray("droppedPackets").getDoubleValue(0));
                            serverHEAReceiveByte.addMetric(heaCaptions, MathUtils.round(physicalPort.getJSONArray("receivedBytes").getDoubleValue(0) / 1024, 2));
                            serverHEASendByte.addMetric(heaCaptions, MathUtils.round(physicalPort.getJSONArray("sentBytes").getDoubleValue(0) / 1024, 2));
                            serverHEATransferByte.addMetric(heaCaptions, MathUtils.round(physicalPort.getJSONArray("transferredBytes").getDoubleValue(0) / 1024, 2));
                            logger.debug("Get PROC HMC[" + vendor.getName() + "]SERVER[" + serverName + "]HEA[" + adapterLocation + ".PORT" + adapterId + "]: <--- " + physicalPort.toJSONString());
                        }
                        serverHEAReceivePackageTotal.addMetric(serverCaptions, physicalPorts.stream().mapToDouble(value -> ((JSONObject) value).getJSONArray("receivedPackets").getDoubleValue(0)).sum());
                        serverHEASendPackageTotal.addMetric(serverCaptions, physicalPorts.stream().mapToDouble(value -> ((JSONObject) value).getJSONArray("sentPackets").getDoubleValue(0)).sum());
                        serverHEADropPackageTotal.addMetric(serverCaptions, physicalPorts.stream().mapToDouble(value -> ((JSONObject) value).getJSONArray("droppedPackets").getDoubleValue(0)).sum());
                        serverHEAReceiveByteTotal.addMetric(serverCaptions, physicalPorts.stream().mapToDouble(value -> ((JSONObject) value).getJSONArray("receivedBytes").getDoubleValue(0)).sum());
                        serverHEASendByteTotal.addMetric(serverCaptions, physicalPorts.stream().mapToDouble(value -> ((JSONObject) value).getJSONArray("sentBytes").getDoubleValue(0)).sum());
                        serverHEATransferByteTotal.addMetric(serverCaptions, physicalPorts.stream().mapToDouble(value -> ((JSONObject) value).getJSONArray("transferredBytes").getDoubleValue(0)).sum());
                    }
                    // VIOS监控指标
                    JSONArray viosUtils = utilSample.getJSONArray("viosUtil");
                    if (viosUtils != null) {
                        for (int i = 0; i < viosUtils.size(); i++) {
                            JSONObject viosUtil = viosUtils.getJSONObject(i);
                            String viosId = viosUtil.getString("uuid");
                            String viosName = viosUtil.getString("name");
                            List<String> viosCaptions = Arrays.asList(String.valueOf(vendor.getId()), viosId, viosName, serverId, serverName);
                            // 处理器监控指标
                            viosCpuMaxCount.addMetric(viosCaptions, viosUtil.getJSONObject("processor").getJSONArray("maxVirtualProcessors").getDoubleValue(0));
                            viosCpuMaxUnit.addMetric(viosCaptions, viosUtil.getJSONObject("processor").getJSONArray("maxProcUnits").getDoubleValue(0));
                            viosCpuEntitledUnit.addMetric(viosCaptions, viosUtil.getJSONObject("processor").getJSONArray("entitledProcUnits").getDoubleValue(0));
                            viosCpuUtilizedUnit.addMetric(viosCaptions, viosUtil.getJSONObject("processor").getJSONArray("utilizedProcUnits").getDoubleValue(0));
                            viosCpuUtilizedCappedUnit.addMetric(viosCaptions, viosUtil.getJSONObject("processor").getJSONArray("utilizedCappedProcUnits").getDoubleValue(0));
                            viosCpuUtilizedUncappedUnit.addMetric(viosCaptions, viosUtil.getJSONObject("processor").getJSONArray("utilizedUncappedProcUnits").getDoubleValue(0));
                            viosCpuIdleUnit.addMetric(viosCaptions, viosUtil.getJSONObject("processor").getJSONArray("idleProcUnits").getDoubleValue(0));
                            viosCpuDonatedUnit.addMetric(viosCaptions, viosUtil.getJSONObject("processor").getJSONArray("donatedProcUnits").getDoubleValue(0));
                            logger.debug("Get PROC HMC[" + vendor.getName() + "]SERVER[" + serverName + "]VIOS[" + viosName + "]CPU: <--- " + viosUtil.getJSONObject("processor").toJSONString());
                            // 内存监控指标
                            viosMemTotal.addMetric(viosCaptions, viosUtil.getJSONObject("memory").getJSONArray("assignedMem").getDoubleValue(0));
                            viosMemUtilized.addMetric(viosCaptions, viosUtil.getJSONObject("memory").getJSONArray("utilizedMem").getDoubleValue(0));
                            logger.debug("Get PROC HMC[" + vendor.getName() + "]SERVER[" + serverName + "]VIOS[" + viosName + "]MEM: <--- " + viosUtil.getJSONObject("memory").toJSONString());
                            // 虚拟网卡监控指标
                            if (viosUtil.containsKey("network")) {
                                JSONArray allAdapters = new JSONArray();
                                if (viosUtil.getJSONObject("network").containsKey("genericAdapters")) {
                                    allAdapters.addAll(viosUtil.getJSONObject("network").getJSONArray("genericAdapters"));
                                }
                                if (viosUtil.getJSONObject("network").containsKey("sharedAdapters")) {
                                    allAdapters.addAll(viosUtil.getJSONObject("network").getJSONArray("sharedAdapters"));
                                }
                                for (int j = 0; j < allAdapters.size(); j++) {
                                    JSONObject adapter = allAdapters.getJSONObject(j);
                                    String adapterId = adapter.getString("id");
                                    String adapterLocation = adapter.getString("physicalLocation");
                                    List<String> viosAdapterCaptions = Arrays.asList(String.valueOf(vendor.getId()), adapterId, adapterLocation, viosId, viosName);
                                    viosNetworkReceivePackage.addMetric(viosAdapterCaptions, adapter.getJSONArray("receivedPackets").getDoubleValue(0));
                                    viosNetworkSendPackage.addMetric(viosAdapterCaptions, adapter.getJSONArray("sentPackets").getDoubleValue(0));
                                    viosNetworkDropPackage.addMetric(viosAdapterCaptions, adapter.getJSONArray("droppedPackets").getDoubleValue(0));
                                    viosNetworkReceiveByte.addMetric(viosAdapterCaptions, MathUtils.round(adapter.getJSONArray("receivedBytes").getDoubleValue(0) / 1024, 2));
                                    viosNetworkSendByte.addMetric(viosAdapterCaptions, MathUtils.round(adapter.getJSONArray("sentBytes").getDoubleValue(0) / 1024, 2));
                                    viosNetworkTransferByte.addMetric(viosAdapterCaptions, MathUtils.round(adapter.getJSONArray("transferredBytes").getDoubleValue(0) / 1024, 2));
                                    logger.debug("Get PROC HMC[" + vendor.getName() + "]SERVER[" + serverName + "]VIOS[" + viosName + "]NETWORK[" + adapterId + "]: <--- " + adapter.toJSONString());
                                }
                                viosNetworkReceivePackageTotal.addMetric(viosCaptions, allAdapters.stream().mapToDouble(value -> ((JSONObject) value).getJSONArray("receivedPackets").getDoubleValue(0)).sum());
                                viosNetworkSendPackageTotal.addMetric(viosCaptions, allAdapters.stream().mapToDouble(value -> ((JSONObject) value).getJSONArray("sentPackets").getDoubleValue(0)).sum());
                                viosNetworkDropPackageTotal.addMetric(viosCaptions, allAdapters.stream().mapToDouble(value -> ((JSONObject) value).getJSONArray("droppedPackets").getDoubleValue(0)).sum());
                                viosNetworkReceiveByteTotal.addMetric(viosCaptions, allAdapters.stream().mapToDouble(value -> ((JSONObject) value).getJSONArray("receivedBytes").getDoubleValue(0)).sum());
                                viosNetworkSendByteTotal.addMetric(viosCaptions, allAdapters.stream().mapToDouble(value -> ((JSONObject) value).getJSONArray("sentBytes").getDoubleValue(0)).sum());
                                viosNetworkTransferByteTotal.addMetric(viosCaptions, allAdapters.stream().mapToDouble(value -> ((JSONObject) value).getJSONArray("transferredBytes").getDoubleValue(0)).sum());
                            }
                            // 存储监控指标
                            if (viosUtil.containsKey("storage")) {
                                JSONArray allStorages = new JSONArray();
                                if (viosUtil.getJSONObject("storage").containsKey("genericPhysicalAdapters")) {
                                    allStorages.addAll(viosUtil.getJSONObject("storage").getJSONArray("genericPhysicalAdapters"));
                                }
                                if (viosUtil.getJSONObject("storage").containsKey("fiberChannelAdapters")) {
                                    allStorages.addAll(viosUtil.getJSONObject("storage").getJSONArray("fiberChannelAdapters"));
                                }
                                if (viosUtil.getJSONObject("storage").containsKey("genericVirtualAdapters")) {
                                    allStorages.addAll(viosUtil.getJSONObject("storage").getJSONArray("genericVirtualAdapters"));
                                }
                                for (int j = 0; j < allStorages.size(); j++) {
                                    JSONObject storage = allStorages.getJSONObject(j);
                                    String storageId = storage.getString("id");
                                    String storageLocation = storage.getString("physicalLocation");
                                    List<String> viosStorageCaptions = Arrays.asList(String.valueOf(vendor.getId()), storageId, storageLocation, viosId, viosName);
                                    viosStorageReadNum.addMetric(viosStorageCaptions, storage.getJSONArray("numOfReads").getDoubleValue(0));
                                    viosStorageWriteNum.addMetric(viosStorageCaptions, storage.getJSONArray("numOfWrites").getDoubleValue(0));
                                    viosStorageReadByte.addMetric(viosStorageCaptions, MathUtils.round(storage.getJSONArray("readBytes").getDoubleValue(0) / 1024, 2));
                                    viosStorageWriteByte.addMetric(viosStorageCaptions, MathUtils.round(storage.getJSONArray("writeBytes").getDoubleValue(0) / 1024, 2));
                                    viosStorageTransmitByte.addMetric(viosStorageCaptions, MathUtils.round(storage.getJSONArray("transmittedBytes").getDoubleValue(0) / 1024, 2));
                                    logger.debug("Get PROC HMC[" + vendor.getName() + "]SERVER[" + serverName + "]VIOS[" + viosName + "]STORAGE[" + storageId + "]: <--- " + storage.toJSONString());
                                }
                                viosStorageReadNumTotal.addMetric(viosCaptions, allStorages.stream().mapToDouble(value -> ((JSONObject) value).getJSONArray("numOfReads").getDoubleValue(0)).sum());
                                viosStorageWriteNumTotal.addMetric(viosCaptions, allStorages.stream().mapToDouble(value -> ((JSONObject) value).getJSONArray("numOfWrites").getDoubleValue(0)).sum());
                                viosStorageReadByteTotal.addMetric(viosCaptions, allStorages.stream().mapToDouble(value -> ((JSONObject) value).getJSONArray("readBytes").getDoubleValue(0)).sum());
                                viosStorageWriteByteTotal.addMetric(viosCaptions, allStorages.stream().mapToDouble(value -> ((JSONObject) value).getJSONArray("writeBytes").getDoubleValue(0)).sum());
                                viosStorageTransmitByteTotal.addMetric(viosCaptions, allStorages.stream().mapToDouble(value -> ((JSONObject) value).getJSONArray("transmittedBytes").getDoubleValue(0)).sum());
                            }
                        }
                    }
                }

                // 解析分区监控数据
                for (Map.Entry<HmcServer, List<Future<String>>> futureListEntry : partitionJSONFutureMap.entrySet()) {
                    for (Future<String> futureEntry : futureListEntry.getValue()) {
                        HmcServer server = futureListEntry.getKey();
                        String serverId = server.getUuid();
                        String serverName = server.getName();

                        JSONObject jsonObject = JSON.parseObject(futureEntry.get());
                        JSONObject utilInfo = jsonObject.getJSONObject("systemUtil").getJSONObject("utilInfo");
                        JSONArray utilSamples = jsonObject.getJSONObject("systemUtil").getJSONArray("utilSamples");
                        // prometheus每次调用只获取最新时间点的数据，所以这里要比对当前获取结果的最大时间
                        JSONObject utilSample = null;
                        for (int i = 0; i < utilSamples.size(); i++) {
                            if (utilSample == null || utilSample.getJSONObject("sampleInfo").getString("timeStamp").compareTo(
                                utilSamples.getJSONObject(i).getJSONObject("sampleInfo").getString("timeStamp")
                            ) < 0) {
                                utilSample = utilSamples.getJSONObject(i);
                            }
                        }
                        // 开始处理
                        JSONObject lparsUtil = utilSample.getJSONArray("lparsUtil").getJSONObject(0);
                        String partitionId = lparsUtil.getString("uuid");
                        String partitionName = lparsUtil.getString("name");
                        List<String> partitionCaptions = Arrays.asList(String.valueOf(vendor.getId()), partitionId, partitionName, serverId, serverName);
                        // 处理器监控指标
                        partitionCpuMaxCount.addMetric(partitionCaptions, lparsUtil.getJSONObject("processor").getJSONArray("maxVirtualProcessors").getDoubleValue(0));
                        partitionCpuMaxUnit.addMetric(partitionCaptions, lparsUtil.getJSONObject("processor").getJSONArray("maxProcUnits").getDoubleValue(0));
                        partitionCpuEntitledUnit.addMetric(partitionCaptions, lparsUtil.getJSONObject("processor").getJSONArray("entitledProcUnits").getDoubleValue(0));
                        partitionCpuUtilizedUnit.addMetric(partitionCaptions, lparsUtil.getJSONObject("processor").getJSONArray("utilizedProcUnits").getDoubleValue(0));
                        partitionCpuUtilizedCappedUnit.addMetric(partitionCaptions, lparsUtil.getJSONObject("processor").getJSONArray("utilizedCappedProcUnits").getDoubleValue(0));
                        partitionCpuUtilizedUncappedUnit.addMetric(partitionCaptions, lparsUtil.getJSONObject("processor").getJSONArray("utilizedUncappedProcUnits").getDoubleValue(0));
                        partitionCpuIdleUnit.addMetric(partitionCaptions, lparsUtil.getJSONObject("processor").getJSONArray("idleProcUnits").getDoubleValue(0));
                        partitionCpuDonatedUnit.addMetric(partitionCaptions, lparsUtil.getJSONObject("processor").getJSONArray("donatedProcUnits").getDoubleValue(0));
                        logger.debug("Get PROC HMC[" + vendor.getName() + "]SERVER[" + serverName + "]PARTITION[" + partitionName + "]CPU: <--- " + lparsUtil.getJSONObject("processor").toJSONString());
                        // 内存监控指标
                        partitionMemLogical.addMetric(partitionCaptions, lparsUtil.getJSONObject("memory").getJSONArray("logicalMem").getDoubleValue(0));
                        partitionMemPhysical.addMetric(partitionCaptions, lparsUtil.getJSONObject("memory").getJSONArray("backedPhysicalMem").getDoubleValue(0));
                        logger.debug("Get PROC HMC[" + vendor.getName() + "]SERVER[" + serverName + "]PARTITION[" + partitionName + "]MEM: <--- " + lparsUtil.getJSONObject("memory").toJSONString());
                        // 存储监控指标
                        if (lparsUtil.containsKey("storage")) {
                            JSONArray allStorages = new JSONArray();
                            if (lparsUtil.getJSONObject("storage").containsKey("genericPhysicalAdapters")) {
                                allStorages.addAll(lparsUtil.getJSONObject("storage").getJSONArray("genericPhysicalAdapters"));
                            }
                            if (lparsUtil.getJSONObject("storage").containsKey("fiberChannelAdapters")) {
                                allStorages.addAll(lparsUtil.getJSONObject("storage").getJSONArray("fiberChannelAdapters"));
                            }
                            if (lparsUtil.getJSONObject("storage").containsKey("genericVirtualAdapters")) {
                                allStorages.addAll(lparsUtil.getJSONObject("storage").getJSONArray("genericVirtualAdapters"));
                            }
                            for (int j = 0; j < allStorages.size(); j++) {
                                JSONObject storage = allStorages.getJSONObject(j);
                                String storageId = storage.getString("id");
                                String storageLocation = storage.getString("physicalLocation");
                                List<String> partitionStorageCaptions = Arrays.asList(String.valueOf(vendor.getId()), storageId, storageLocation, partitionId, partitionName);
                                partitionStorageReadNum.addMetric(partitionStorageCaptions, storage.getJSONArray("numOfReads").getDoubleValue(0));
                                partitionStorageWriteNum.addMetric(partitionStorageCaptions, storage.getJSONArray("numOfWrites").getDoubleValue(0));
                                partitionStorageReadByte.addMetric(partitionStorageCaptions, MathUtils.round(storage.getJSONArray("readBytes").getDoubleValue(0) / 1024, 2));
                                partitionStorageWriteByte.addMetric(partitionStorageCaptions, MathUtils.round(storage.getJSONArray("writeBytes").getDoubleValue(0) / 1024, 2));
                                partitionStorageTransmitByte.addMetric(partitionStorageCaptions, MathUtils.round(storage.getJSONArray("transmittedBytes").getDoubleValue(0) / 1024, 2));
                                logger.debug("Get PROC HMC[" + vendor.getName() + "]SERVER[" + serverName + "]PARTITION[" + partitionName + "]STORAGE[" + storageId + "]: <--- " + storage.toJSONString());
                            }
                            partitionStorageReadNumTotal.addMetric(partitionCaptions, allStorages.stream().mapToDouble(value -> ((JSONObject) value).getJSONArray("numOfReads").getDoubleValue(0)).sum());
                            partitionStorageWriteNumTotal.addMetric(partitionCaptions, allStorages.stream().mapToDouble(value -> ((JSONObject) value).getJSONArray("numOfWrites").getDoubleValue(0)).sum());
                            partitionStorageReadByteTotal.addMetric(partitionCaptions, allStorages.stream().mapToDouble(value -> ((JSONObject) value).getJSONArray("readBytes").getDoubleValue(0)).sum());
                            partitionStorageWriteByteTotal.addMetric(partitionCaptions, allStorages.stream().mapToDouble(value -> ((JSONObject) value).getJSONArray("writeBytes").getDoubleValue(0)).sum());
                            partitionStorageTransmitByteTotal.addMetric(partitionCaptions, allStorages.stream().mapToDouble(value -> ((JSONObject) value).getJSONArray("transmittedBytes").getDoubleValue(0)).sum());
                        }
                    }
                }
            } catch (SAXException | IOException | ExecutionException e) {
                logger.error("Get PROC HMC[" + vendor.getName() + "] ProcessorMonitor error:{} ", e.getMessage(), e);
            } catch (InterruptedException e) {
                Thread.currentThread().interrupt();
                logger.error("Interrupted PROC HMC[" + vendor.getName() + "] LongTermMonitor");
                break;
            }
        }
        return Arrays.asList(
                serverCpuTotal, serverCpuAvailable, serverCpuUsed, serverCpuUtilized,
                serverMemTotal, serverMemAvailable, serverMemUsed, serverMemPartition, serverMemSystem,
                serverShareCpuPoolAssigned, serverShareCpuPoolAvailable, serverShareCpuPoolBorrowed, serverShareCpuPoolConfigured, serverShareCpuPoolUtilized,
                serverHEAReceivePackage, serverHEASendPackage, serverHEADropPackage, serverHEAReceiveByte, serverHEASendByte, serverHEATransferByte,
                serverHEAReceivePackageTotal, serverHEASendPackageTotal, serverHEADropPackageTotal, serverHEAReceiveByteTotal, serverHEASendByteTotal, serverHEATransferByteTotal,
                viosCpuMaxCount, viosCpuMaxUnit, viosCpuEntitledUnit, viosCpuUtilizedUnit, viosCpuUtilizedCappedUnit, viosCpuUtilizedUncappedUnit, viosCpuIdleUnit, viosCpuDonatedUnit,
                viosMemTotal, viosMemUtilized,
                viosNetworkReceivePackage, viosNetworkSendPackage, viosNetworkDropPackage, viosNetworkReceiveByte, viosNetworkSendByte, viosNetworkTransferByte,
                viosNetworkReceivePackageTotal, viosNetworkSendPackageTotal, viosNetworkDropPackageTotal, viosNetworkReceiveByteTotal, viosNetworkSendByteTotal, viosNetworkTransferByteTotal,
                viosStorageReadByte, viosStorageWriteByte, viosStorageReadNum, viosStorageWriteNum, viosStorageTransmitByte,
                viosStorageReadByteTotal, viosStorageWriteByteTotal, viosStorageReadNumTotal, viosStorageWriteNumTotal, viosStorageTransmitByteTotal,
                partitionCpuMaxCount, partitionCpuMaxUnit, partitionCpuEntitledUnit, partitionCpuUtilizedUnit, partitionCpuUtilizedCappedUnit, partitionCpuUtilizedUncappedUnit, partitionCpuIdleUnit, partitionCpuDonatedUnit,
                partitionMemLogical, partitionMemPhysical,
                partitionStorageReadByte, partitionStorageWriteByte, partitionStorageReadNum, partitionStorageWriteNum, partitionStorageTransmitByte,
                partitionStorageReadByteTotal, partitionStorageWriteByteTotal, partitionStorageReadNumTotal, partitionStorageWriteNumTotal, partitionStorageTransmitByteTotal
        );
    }

    @Scheduled(initialDelay = 10000L, fixedRate = 60000)
    @Async
    @Override
    public void scheduledCollect() {
        if (!currentService.isLeader()) {
            logger.info(this.getClass().getName() + "当前节点是否是主节点:" + currentService.isLeader());
            return;
        }
        logger.info("Collect HMC metrics start...");
        long start = System.currentTimeMillis();
        // 获取平台数据
        long vendorStart = System.currentTimeMillis();
        BocloudResult bocloudResult = cmpCloudVendorService.list(CloudProvider.HMC);
        if (bocloudResult.isFailed()) {
            logger.error("Get vendor list failed, error:{}", bocloudResult.getMessage());
            return;
        }
        logger.debug("Get vendor list end, use:" + (System.currentTimeMillis() - vendorStart) / 1000D + "s");
        JSONObject data = (JSONObject) bocloudResult.getData();
        List<CloudVendor> allVendors = data.getJSONArray(Common.ROWS).toJavaList(CloudVendor.class);

        // 获取每一个平台的sessionId和主机
        List<CloudVendor> preparedVendors = new ArrayList<>(allVendors);
        Map<String, String> sessionIds = new HashMap<>();
        MultiValueMap<String, HmcServer> servers = new LinkedMultiValueMap<>();
        MultiValueMap<String, HmcPartition> vioses = new LinkedMultiValueMap<>();
        List<GaugeMetricFamily> AllMetrics = new ArrayList<>();;

        // 执行登录
        Map<CloudVendor, Future<String>> sessionFutures = new HashMap<>();
        for (CloudVendor vendor : preparedVendors) {
            Future<String> future = executorService.submit(() -> {
                String address = JSONObject.parseObject(vendor.getAuthentication()).getString("address");
                String account = JSONObject.parseObject(vendor.getAuthentication()).getString("account");
                String password = JSONObject.parseObject(vendor.getAuthentication()).getString("password");
                password = new DESEncrptor().decrypt(password.trim(), "BocloudCMP");
                return login(address, account, password);
            });
            sessionFutures.put(vendor, future);
        }
        for (Map.Entry<CloudVendor, Future<String>> entry : sessionFutures.entrySet()) {
            try {
                sessionIds.put(String.valueOf(entry.getKey().getId()), entry.getValue().get());
            } catch (Exception e) {
                logger.error("Prepare vendor[" + entry.getKey().getName() + "] failed, error:{}", e.getMessage(), e);
                preparedVendors.remove(entry.getKey());
            }
        }
        try {
            // 查询所有主机
            Map<CloudVendor, Future<List<HmcServer>>> serverFutures = new HashMap<>();
            for (CloudVendor vendor : preparedVendors) {
                String address = JSONObject.parseObject(vendor.getAuthentication()).getString("address");
                Future<List<HmcServer>> future = executorService.submit(() -> getServers(address, sessionIds.get(String.valueOf(vendor.getId()))));
                serverFutures.put(vendor, future);
            }
            for (Map.Entry<CloudVendor, Future<List<HmcServer>>> entry : serverFutures.entrySet()) {
                try {
                    servers.put(String.valueOf(entry.getKey().getId()), entry.getValue().get());
                } catch (Exception e) {
                    logger.error("Prepare vendor[" + entry.getKey().getName() + "] failed, error:{}", e.getMessage(), e);
                    preparedVendors.remove(entry.getKey());
                }
            }
            // 自动为主机开启监控
            Map<HmcServer, Future<String>> preferenceFutures = new HashMap<>();
            for (CloudVendor vendor : preparedVendors) {
                for (HmcServer server : servers.get(String.valueOf(vendor.getId()))) {
                    String address = JSONObject.parseObject(vendor.getAuthentication()).getString("address");
                    Future<String> future = executorService.submit(() -> setPreferences(address, server.getUuid(), sessionIds.get(String.valueOf(vendor.getId()))));
                    preferenceFutures.put(server, future);
                }
            }
            for (CloudVendor vendor : new ArrayList<>(preparedVendors)) {
                try {
                    for (HmcServer server : servers.get(String.valueOf(vendor.getId()))) {
                        preferenceFutures.get(server).get();
                    }
                } catch (Exception e) {
                    logger.error("Prepare vendor[" + vendor.getName() + "] failed, error:{}", e.getMessage(), e);
                    preparedVendors.remove(vendor);
                }
            }
            // 查询所有vios
            Map<HmcServer, Future<List<HmcPartition>>> viosFutures = new HashMap<>();
            for (CloudVendor vendor : preparedVendors) {
                for (HmcServer server : servers.get(String.valueOf(vendor.getId()))) {
                    String address = JSONObject.parseObject(vendor.getAuthentication()).getString("address");
                    Future<List<HmcPartition>> future = executorService.submit(() -> getVioses(address, server.getUuid(), sessionIds.get(String.valueOf(vendor.getId()))));
                    viosFutures.put(server, future);
                }
            }
            for (CloudVendor vendor : new ArrayList<>(preparedVendors)) {
                try {
                    for (HmcServer server : servers.get(String.valueOf(vendor.getId()))) {
                        vioses.put(server.getUuid(), viosFutures.get(server).get());
                    }
                } catch (Exception e) {
                    logger.error("Prepare vendor[" + vendor.getName() + "] failed, error:{}", e.getMessage(), e);
                    preparedVendors.remove(vendor);
                }
            }

            if (threadPoolSize > 2) {
                Future<List<GaugeMetricFamily>> ltmFuture = executorService.submit(() -> longTermMonitor(preparedVendors, sessionIds, servers, vioses));
                Future<List<GaugeMetricFamily>> procFuture = executorService.submit(() -> processorMonitor(preparedVendors, sessionIds, servers, vioses));
                try {
                    AllMetrics.addAll(ltmFuture.get());
                    AllMetrics.addAll(procFuture.get());
                } catch (ExecutionException e) {
                    logger.error("Collect HMC metrics error:{} ", e.getMessage(), e);
                } catch (InterruptedException e) {
                    Thread.currentThread().interrupt();
                    logger.warn("Interrupted HMC metrics");
                }
            } else {
                try {
                    AllMetrics.addAll(longTermMonitor(preparedVendors, sessionIds, servers, vioses));
                } catch (ParserConfigurationException e) {
                    logger.error("Collect HMC metrics error:{} ", e.getMessage(), e);
                }
                try {
                    AllMetrics.addAll(processorMonitor(preparedVendors, sessionIds, servers, vioses));
                } catch (ParserConfigurationException e) {
                    logger.error("Collect HMC metrics error:{} ", e.getMessage(), e);
                }
            }
        } finally {
            for (CloudVendor vendor : allVendors) {
                if (sessionIds.containsKey(String.valueOf(vendor.getId()))) {
                    String address = JSONObject.parseObject(vendor.getAuthentication()).getString("address");
                    logout(address, sessionIds.get(String.valueOf(vendor.getId())));
                }
            }
        }

        cache.put(TIME, new Date());
        cache.put(DATA, AllMetrics);

        logger.info("Collect HMC metrics end! use " + (System.currentTimeMillis() - start) / 1000D + "s");
    }

    @Override
    protected Map<String, Object> getCache() {
        return cache;
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        register();
    }

    private class Entry {
        private final String id;
        private final String category;
        private final String href;
        private final String date;

        public Entry(String id, String category, String href, String date) {
            this.id = id;
            this.category = category;
            this.href = href;
            this.date = date;
        }

        public String getId() {
            return id;
        }

        public String getCategory() {
            return category;
        }

        public String getHref() {
            return href;
        }

        public String getDate() {
            return date;
        }
    }

    private class Preference {
        private String energyMonitoringCapable;
        private String longTermMonitorEnabled;
        private String aggregationEnabled;
        private String shortTermMonitorEnabled;
        private String computeLTMEnabled;
        private String energyMonitorEnabled;

        public String getEnergyMonitoringCapable() {
            return energyMonitoringCapable;
        }

        public void setEnergyMonitoringCapable(String energyMonitoringCapable) {
            this.energyMonitoringCapable = energyMonitoringCapable;
        }

        public String getLongTermMonitorEnabled() {
            return longTermMonitorEnabled;
        }

        public void setLongTermMonitorEnabled(String longTermMonitorEnabled) {
            this.longTermMonitorEnabled = longTermMonitorEnabled;
        }

        public String getAggregationEnabled() {
            return aggregationEnabled;
        }

        public void setAggregationEnabled(String aggregationEnabled) {
            this.aggregationEnabled = aggregationEnabled;
        }

        public String getShortTermMonitorEnabled() {
            return shortTermMonitorEnabled;
        }

        public void setShortTermMonitorEnabled(String shortTermMonitorEnabled) {
            this.shortTermMonitorEnabled = shortTermMonitorEnabled;
        }

        public String getComputeLTMEnabled() {
            return computeLTMEnabled;
        }

        public void setComputeLTMEnabled(String computeLTMEnabled) {
            this.computeLTMEnabled = computeLTMEnabled;
        }

        public String getEnergyMonitorEnabled() {
            return energyMonitorEnabled;
        }

        public void setEnergyMonitorEnabled(String energyMonitorEnabled) {
            this.energyMonitorEnabled = energyMonitorEnabled;
        }
    }
}
