package com.bocloud.mcs.collector.huawei;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.encrypt.DESEncrptor;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.utils.Common;
import com.bocloud.ims.entity.CloudVendor;
import com.bocloud.ims.enums.CloudProvider;
import com.bocloud.mcs.collector.BocloudCollector;
import com.bocloud.service.core.CurrentService;
import com.huaweicloud.sdk.ces.v1.CesClient;
import com.huaweicloud.sdk.ces.v1.model.*;
import com.huaweicloud.sdk.ces.v1.region.CesRegion;
import com.huaweicloud.sdk.core.ClientBuilder;
import com.huaweicloud.sdk.core.auth.BasicCredentials;
import com.huaweicloud.sdk.core.auth.ICredential;
import com.huaweicloud.sdk.ecs.v2.EcsClient;
import com.huaweicloud.sdk.ecs.v2.model.ListServersDetailsRequest;
import com.huaweicloud.sdk.ecs.v2.model.ListServersDetailsResponse;
import com.huaweicloud.sdk.ecs.v2.model.ServerDetail;
import com.huaweicloud.sdk.ecs.v2.region.EcsRegion;
import io.prometheus.client.GaugeMetricFamily;
import org.apache.commons.lang3.StringUtils;
import org.apache.http.ssl.SSLContextBuilder;
import org.openstack4j.core.transport.Config;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.scheduling.annotation.Async;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;
import org.springframework.util.Assert;

import java.security.NoSuchAlgorithmException;
import java.time.LocalDateTime;
import java.time.ZoneOffset;
import java.util.*;
import java.util.concurrent.ConcurrentHashMap;
import java.util.stream.Collectors;

/**
 * Huawei collector
 *
 * @author zhangyf
 * @since 2021-01-15
 */
@Component
public class HuaweiVmCollector extends BocloudCollector implements InitializingBean {

    private static final Logger logger = LoggerFactory.getLogger(HuaweiVmCollector.class);

    private static SSLContextBuilder contextBuilder;
    private static Config config = null;

    @Autowired
    private CurrentService currentService;

    public final static Map<String, String> METRIC_NAME_MAP = new HashMap();
    public final static Map<String, String> AGENT_METRIC_NAME_MAP = new HashMap();

    static {
        // 基础监控SYS.ECS
        METRIC_NAME_MAP.put("cpu_util", "CPU使用率");
        METRIC_NAME_MAP.put("mem_util", "内存使用率");
        METRIC_NAME_MAP.put("disk_util_inband", "磁盘使用率");
        METRIC_NAME_MAP.put("disk_read_bytes_rate", "磁盘读带宽");
        METRIC_NAME_MAP.put("disk_write_bytes_rate", "磁盘写带宽");
        METRIC_NAME_MAP.put("disk_read_requests_rate", "磁盘读IOPS");
        METRIC_NAME_MAP.put("disk_write_requests_rate", "磁盘写IOPS");
        METRIC_NAME_MAP.put("network_incoming_bytes_rate_inband", "带内网络流入速率");
        METRIC_NAME_MAP.put("network_outgoing_bytes_rate_inband", "带内网络流出速率");
        METRIC_NAME_MAP.put("network_vm_connections", "网络连接数");
        METRIC_NAME_MAP.put("network_outgoing_bytes_aggregate_rate", "带外网络流出速率");
        METRIC_NAME_MAP.put("network_incoming_bytes_aggregate_rate", "带内网络流入速率");
        METRIC_NAME_MAP.put("network_vm_bandwidth_in", "虚拟机入方向带宽");
        METRIC_NAME_MAP.put("network_vm_bandwidth_out", "虚拟机出方向带宽");
        // Agent监控AGT.ECS
        AGENT_METRIC_NAME_MAP.put("cpu_usage", "CPU利用率");
        AGENT_METRIC_NAME_MAP.put("mem_usedPercent", "内存利用率");
    }

    /**
     * 本地缓存
     */
    Map<String, Object> cache = new HashMap<>();

    @Value("${collector.huawei.vm.enable:true}")
    boolean enable;
    private String prefix = "huawei_";

    static {
        contextBuilder = new SSLContextBuilder();
        // 全部信任 不做身份鉴定
        try {
            contextBuilder.loadTrustMaterial(null, (x509Certificates, s) -> true);
            config = Config.newConfig()
                    .withConnectionTimeout(50000)
                    .withReadTimeout(50000)
                    .withSSLVerificationDisabled()
                    .withSSLContext(contextBuilder.build());
        } catch (NoSuchAlgorithmException e) {
            logger.error("NoSuchAlgorithmException", e);
        } catch (Exception e) {
            logger.error("KeyStoreException", e);
        }
    }

    @Scheduled(initialDelay = 2000L, fixedRate = 120000L)
    @Async
    @Override
    public void scheduledCollect() {
        if (!enable) {
            return;
        }
        if (!currentService.isLeader()) {
            logger.info(this.getClass().getName() + "当前节点是否是主节点:" + currentService.isLeader());
            return;
        }
        Date now = new Date();
        logger.info("collect huawei vm metrics start...");
        GaugeMetricFamily collectTimeUse = new GaugeMetricFamily(prefix + "collect_time_use", "help",
                Collections.singletonList("type"));
        BocloudResult result = cmpCloudVendorService.list(CloudProvider.HUAWEI);
        Assert.isTrue(result.isSuccess(), "get cloud vendor list failed , error:" + result.getMessage());
        JSONObject data = (JSONObject) result.getData();
        List<CloudVendor> vendors = data.getJSONArray(Common.ROWS).toJavaList(CloudVendor.class);

        Map<String, GaugeMetricFamily> map = new ConcurrentHashMap<>();
        for (CloudVendor vendor : vendors) {
            try {
                String accessKey = JSONObject.parseObject(vendor.getAuthentication()).getString("accessKey");
                String secretKey = JSONObject.parseObject(vendor.getAuthentication()).getString("secretKey");
                accessKey = new DESEncrptor().decrypt(accessKey.trim(), "BocloudCMP");
                secretKey = new DESEncrptor().decrypt(secretKey.trim(), "BocloudCMP");
                List<String> regionList = JSONObject.parseObject(vendor.getAuthentication()).getJSONArray("regions").toJavaList(String.class);
                for (String regionId : regionList) {
                    ICredential auth = new BasicCredentials().withAk(accessKey).withSk(secretKey);
                    EcsClient ecsClient = new ClientBuilder<>(EcsClient::new, BasicCredentials.class.getSimpleName())
                            .withCredential(auth)
                            .withRegion(EcsRegion.valueOf(regionId))
                            .build();
                    ListServersDetailsResponse response;
                    int offset = 1;
                    do {
                        ListServersDetailsRequest request = new ListServersDetailsRequest()
                                .withLimit(1000)
                                .withOffset(offset)
                                .withStatus("ACTIVE");
                        offset = offset + 1000;
                        response = ecsClient.listServersDetails(request);
                        List<String> instanceIds = Optional.ofNullable(response.getServers()).orElse(new ArrayList<>())
                                .stream().map(ServerDetail::getId).collect(Collectors.toList());

                        CesClient cesClient = new ClientBuilder<>(CesClient::new, BasicCredentials.class.getSimpleName())
                                .withCredential(auth)
                                .withRegion(CesRegion.valueOf(regionId))
                                .build();

                        Optional.ofNullable(instanceIds).orElse(new ArrayList<>()).forEach(instanceId -> {
                            ListMetricsRequest projectRequest = new ListMetricsRequest();
                            projectRequest.setDim0("instance_id," + instanceId);
                            ListMetricsResponse projectResponse = cesClient.listMetrics(projectRequest);
                            if (projectResponse == null || projectResponse.getMetrics().size() == 0) {
                                return;
                            }
                            List<MetricsDimension> listMetricsDimensions = new ArrayList<>();
                            listMetricsDimensions.add(new MetricsDimension().withName("instance_id").withValue(instanceId));
                            // 获取所有metric信息并过滤需要指标
                            List<MetricInfoList> metrics = projectResponse.getMetrics();
                            List<MetricInfo> metricInfoList = Optional.ofNullable(metrics).orElse(new ArrayList<>()).stream()
                                    .filter(metric -> (StringUtils.startsWithIgnoreCase(metric.getMetricName(), "SlAsH") && (StringUtils.endsWith(metric.getMetricName(), "disk_usedPercent")
                                            || StringUtils.endsWith(metric.getMetricName(), "disk_free")))
                                            || METRIC_NAME_MAP.containsKey(metric.getMetricName())
                                            || AGENT_METRIC_NAME_MAP.containsKey(metric.getMetricName()))
                                    .map(metric -> new MetricInfo().withNamespace(metric.getNamespace()).withMetricName(metric.getMetricName()).withDimensions(listMetricsDimensions))
                                    .collect(Collectors.toList());

                            int size = metricInfoList.size();
                            int i = 0;
                            while (size > 0) {
                                size -= 10;
                                List<MetricInfo> metricModels = metricInfoList.stream().skip(10 * i).limit(10).collect(Collectors.toList());
                                i++;
                                BatchListMetricDataRequest batchListMetricDataRequest = new BatchListMetricDataRequest();
                                BatchListMetricDataRequestBody body = new BatchListMetricDataRequestBody();

                                LocalDateTime localDateTime = LocalDateTime.now();
                                body.withTo(localDateTime.toInstant(ZoneOffset.of("+8")).toEpochMilli());
                                body.withFrom(localDateTime.minusMinutes(10).toInstant(ZoneOffset.of("+8")).toEpochMilli());
                                body.withFilter("average");
                                body.withPeriod("1");
                                body.withMetrics(metricModels);
                                batchListMetricDataRequest.withBody(body);
                                BatchListMetricDataResponse batchListMetricDataResponse = cesClient.batchListMetricData(batchListMetricDataRequest);
                                batchListMetricDataResponse.getMetrics().forEach(batchMetricData -> {
                                    if (batchMetricData.getDatapoints().isEmpty()) {
                                        return;
                                    }
                                    String namespace = batchMetricData.getNamespace().replace(".", "_").toLowerCase();
                                    if (StringUtils.startsWith(batchMetricData.getMetricName(), "SlAsH")) {
                                        // 获取到磁盘目录名称
                                        String substring = batchMetricData.getMetricName().substring(0, batchMetricData.getMetricName().indexOf("_"));
                                        String diskName = substring.replaceAll("SlAsH", "");

                                        GaugeMetricFamily gauge = map.get(batchMetricData.getNamespace() + batchMetricData.getMetricName());
                                        if (gauge == null) {
                                            String metricName = batchMetricData.getMetricName().substring(batchMetricData.getMetricName().indexOf("_") + 1);
                                            gauge = new GaugeMetricFamily(prefix + namespace + "_" + metricName, "help", Arrays.asList("instance_id", "path", "vendor_id"));
                                            map.put(batchMetricData.getNamespace() + batchMetricData.getMetricName(), gauge);
                                        }
                                        Double value = batchMetricData.getDatapoints().get(batchMetricData.getDatapoints().size() - 1).getAverage();
                                        gauge.addMetric(Arrays.asList(batchMetricData.getDimensions().get(0).getValue(), "/" + diskName, String.valueOf(vendor.getId())), value);
                                    } else {
                                        GaugeMetricFamily gauge = map.get(batchMetricData.getNamespace() + batchMetricData.getMetricName());
                                        if (gauge == null) {
                                            gauge = new GaugeMetricFamily(prefix + namespace + "_" + batchMetricData.getMetricName(), "help", Arrays.asList("instance_id", "vendor_id"));
                                            map.put(batchMetricData.getNamespace() + batchMetricData.getMetricName(), gauge);
                                        }
                                        Double value = batchMetricData.getDatapoints().get(batchMetricData.getDatapoints().size() - 1).getAverage();
                                        gauge.addMetric(Arrays.asList(batchMetricData.getDimensions().get(0).getValue(), String.valueOf(vendor.getId())), value);
                                    }
                                });
                            }
                        });
                    } while (response.getServers().size() > 0);
                }
            } catch (RuntimeException e) {
                logger.error("invoke huawei vm get metrics error,", e);
            }
        }
        List<MetricFamilySamples> mfs = new ArrayList<>(map.values());
        collectTimeUse.addMetric(Collections.singletonList("huawei_vm"), (System.currentTimeMillis() - now.getTime()));
        mfs.add(collectTimeUse);
        cache.put(TIME, new Date());
        cache.put(DATA, mfs);
        logger.info("collect huawei vm metrics end! use " + (System.currentTimeMillis() - now.getTime()) / 1000 + "s");
    }

    @Override
    public Map<String, Object> getCache() {
        return this.cache;
    }

}