package com.bocloud.mcs.collector.openstack;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.encrypt.DESEncrptor;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.utils.Common;
import com.bocloud.ims.entity.CloudVendor;
import com.bocloud.ims.enums.CloudProvider;
import com.bocloud.mcs.collector.BocloudCollector;
import com.bocloud.mcs.common.Constants;
import com.bocloud.service.core.CurrentService;
import io.prometheus.client.GaugeMetricFamily;
import org.apache.http.ssl.SSLContextBuilder;
import org.openstack4j.api.OSClient;
import org.openstack4j.core.transport.Config;
import org.openstack4j.model.common.Identifier;
import org.openstack4j.model.compute.ext.Hypervisor;
import org.openstack4j.openstack.OSFactory;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.scheduling.annotation.Async;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import java.security.KeyManagementException;
import java.security.KeyStoreException;
import java.security.NoSuchAlgorithmException;
import java.util.*;

@Component
public class HypervisorCollector extends BocloudCollector {

    private static final Logger logger = LoggerFactory.getLogger(HypervisorCollector.class);

    protected static Config config;

    static {
        // 全部信任 不做身份鉴定
        try {
            SSLContextBuilder contextBuilder = SSLContextBuilder.create().loadTrustMaterial(null,
                    (x509Certificates, authType) -> true);
            config = Config.newConfig().withSSLVerificationDisabled().withSSLContext(contextBuilder.build())
                    .withReadTimeout(20 * 1000 * 1000).withConnectionTimeout(20 * 1000 * 1000);
        } catch (NoSuchAlgorithmException e) {
            logger.error("NoSuchAlgorithmException", e);
        } catch (KeyStoreException e) {
            logger.error("KeyStoreException", e);
        } catch (KeyManagementException e) {
            logger.error("KeyManagementException", e);
        }
    }

    Map<String, Object> cache = new HashMap<>();

    @Autowired
    private CurrentService currentService;

    @Value("${collector.openstack.hypervisor.enable:true}")
    boolean enable;

    @Override
    public List<MetricFamilySamples> collect() {
        return super.collect();
    }

    @Scheduled(initialDelay = 3000L, fixedRate = 60000L)
    @Async
    @Override
    public void scheduledCollect() {
        if (!currentService.isLeader()) {
            logger.info(this.getClass().getName() + "当前节点是否是主节点:" + currentService.isLeader());
            return;
        }
        Date now = new Date();
        logger.info("collect hypervisor metrics start...");
        BocloudResult bocloudResult = cmpCloudVendorService.list(CloudProvider.OPENSTACK);
        if (bocloudResult.isFailed()) {
            logger.error("get cloud vendor list failed , error:{}", bocloudResult.getMessage());
            return;
        }
        JSONObject data = (JSONObject) bocloudResult.getData();
        List<CloudVendor> listCloudVendor = data.getJSONArray(Common.ROWS).toJavaList(CloudVendor.class);

        List<String> labels = Arrays.asList("vendor_id", "id", "hypervisor_hostname", "host_ip");
        GaugeMetricFamily vcpusTotal = new GaugeMetricFamily(Constants.PREFIX + "hypervisor_vcpus_total", "help",
                labels);
        GaugeMetricFamily vcpusUsed = new GaugeMetricFamily(Constants.PREFIX + "hypervisor_vcpus_used", "help", labels);
        GaugeMetricFamily memTotal = new GaugeMetricFamily(Constants.PREFIX + "hypervisor_memory_mbs_total", "help",
                labels);
        GaugeMetricFamily memUsed = new GaugeMetricFamily(Constants.PREFIX + "hypervisor_memory_mbs_used", "help",
                labels);
        GaugeMetricFamily diskTotal = new GaugeMetricFamily(Constants.PREFIX + "hypervisor_disk_gbs_total", "help",
                labels);
        GaugeMetricFamily diskUsed = new GaugeMetricFamily(Constants.PREFIX + "hypervisor_disk_gbs_used", "help",
                labels);
        GaugeMetricFamily leastDiskAvailable = new GaugeMetricFamily(
                Constants.PREFIX + "hypervisor_least_disk_available", "help", labels);
        GaugeMetricFamily runningVms = new GaugeMetricFamily(Constants.PREFIX + "hypervisor_running_vms", "help",
                labels);
        GaugeMetricFamily currentWorkload = new GaugeMetricFamily(Constants.PREFIX + "hypervisor_current_workload",
                "help", labels);
        GaugeMetricFamily status = new GaugeMetricFamily(Constants.PREFIX + "host_success", "help",
                labels);

        GaugeMetricFamily collectTimeUse = new GaugeMetricFamily(Constants.PREFIX + "collect_time_use", "help",
                Arrays.asList("type"));
        List<MetricFamilySamples> mfs = new ArrayList<>();
        for (CloudVendor cloudVendor : listCloudVendor) {
            try {
                OSClient.OSClientV3 os;
                if ("V3".equals(cloudVendor.getVersion())) {
                    String address = JSONObject.parseObject(cloudVendor.getAuthentication()).getString("address");
                    String account = JSONObject.parseObject(cloudVendor.getAuthentication()).getString("account");
                    String password = JSONObject.parseObject(cloudVendor.getAuthentication()).getString("password");
                    String domainName = JSONObject.parseObject(cloudVendor.getAuthentication()).getString("domainName");
                    String projectName = JSONObject.parseObject(cloudVendor.getAuthentication()).getString("projectName");
                    password = new DESEncrptor().decrypt(password.trim(), "BocloudCMP");
                    Identifier domainIdentifier = Identifier.byName(domainName);
                    os = OSFactory.builderV3().endpoint(address.trim() + "/v3")
                            .credentials(account, password, domainIdentifier)
                            .scopeToProject(Identifier.byName(projectName), domainIdentifier)
                            .withConfig(config)
                            .authenticate();
                } else {
                    logger.error("unsupported version {}", cloudVendor.getVersion());
                    break;
                }
                List<? extends Hypervisor> hypervisors = os.compute().hypervisors().list();
                logger.info("get {} hypervisors ", hypervisors.size());
                // server
                for (Hypervisor hypervisor : hypervisors) {
                    logger.debug("collect hypervisor:{} metrics", hypervisor.getId());
                    vcpusTotal.addMetric(Arrays.asList(String.valueOf(cloudVendor.getId()), hypervisor.getId(),
                            hypervisor.getHypervisorHostname(), hypervisor.getHostIP()), hypervisor.getVirtualCPU());
                    vcpusUsed.addMetric(Arrays.asList(String.valueOf(cloudVendor.getId()), hypervisor.getId(),
                            hypervisor.getHypervisorHostname(), hypervisor.getHostIP()),
                            hypervisor.getVirtualUsedCPU());
                    memTotal.addMetric(Arrays.asList(String.valueOf(cloudVendor.getId()), hypervisor.getId(),
                            hypervisor.getHypervisorHostname(), hypervisor.getHostIP()), hypervisor.getLocalMemory());
                    memUsed.addMetric(Arrays.asList(String.valueOf(cloudVendor.getId()), hypervisor.getId(),
                            hypervisor.getHypervisorHostname(), hypervisor.getHostIP()),
                            hypervisor.getLocalMemoryUsed());
                    diskTotal.addMetric(Arrays.asList(String.valueOf(cloudVendor.getId()), hypervisor.getId(),
                            hypervisor.getHypervisorHostname(), hypervisor.getHostIP()), hypervisor.getLocalDisk());
                    diskUsed.addMetric(Arrays.asList(String.valueOf(cloudVendor.getId()), hypervisor.getId(),
                            hypervisor.getHypervisorHostname(), hypervisor.getHostIP()), hypervisor.getLocalDiskUsed());
                    leastDiskAvailable.addMetric(Arrays.asList(String.valueOf(cloudVendor.getId()), hypervisor.getId(),
                            hypervisor.getHypervisorHostname(), hypervisor.getHostIP()),
                            hypervisor.getLeastDiskAvailable());
                    runningVms.addMetric(Arrays.asList(String.valueOf(cloudVendor.getId()), hypervisor.getId(),
                            hypervisor.getHypervisorHostname(), hypervisor.getHostIP()), hypervisor.getRunningVM());
                    currentWorkload.addMetric(Arrays.asList(String.valueOf(cloudVendor.getId()), hypervisor.getId(),
                            hypervisor.getHypervisorHostname(), hypervisor.getHostIP()),
                            hypervisor.getCurrentWorkload());
                    // @version 5.6.0-SNAPSHOT添加状态告警
                    String state = hypervisor.getState();
                    int stateMonitor = "up".equalsIgnoreCase(state) ? 1 : 0;
                    status.addMetric(Arrays.asList(String.valueOf(cloudVendor.getId()), hypervisor.getId(),
                            hypervisor.getHypervisorHostname(), hypervisor.getHostIP()),
                            stateMonitor);
                }
            } catch (Exception e) {
                logger.error("collect hypervisor metrics error!" + e.getMessage(), e);
            }
        }
        collectTimeUse.addMetric(Arrays.asList("hypervisor"), (System.currentTimeMillis() - now.getTime()));
        mfs.add(vcpusTotal);
        mfs.add(vcpusUsed);
        mfs.add(memTotal);
        mfs.add(memUsed);
        mfs.add(diskTotal);
        mfs.add(diskUsed);
        mfs.add(leastDiskAvailable);
        mfs.add(runningVms);
        mfs.add(currentWorkload);
        mfs.add(status);
        mfs.add(collectTimeUse);
        cache.put(TIME, new Date());
        cache.put(DATA, mfs);
        logger.info("collect hypervisor metrics end! use " + (System.currentTimeMillis() - now.getTime()) / 1000 + "s");
    }

    @Override
    public Map<String, Object> getCache() {
        return this.cache;
    }

}
