package com.bocloud.mcs.collector.tianyicloud.core;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.PropertyNamingStrategy;
import com.alibaba.fastjson.serializer.SerializeConfig;
import com.bocloud.common.enums.PostDataFormat;
import com.bocloud.common.http.HttpClient;
import com.bocloud.common.model.Result;
import com.bocloud.common.utils.MapTools;
import com.bocloud.mcs.collector.tianyicloud.Util;
import com.bocloud.mcs.collector.tianyicloud.model.Dimension;
import com.bocloud.mcs.collector.tianyicloud.model.MetricModel;
import com.bocloud.mcs.collector.tianyicloud.model.MetricWithValueModel;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.NoArgsConstructor;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.util.Assert;

import java.time.LocalDateTime;
import java.time.ZoneOffset;
import java.util.*;
import java.util.stream.Collectors;

@Data
@AllArgsConstructor
@NoArgsConstructor
public class TianyiClient {
    private static final Logger logger = LoggerFactory.getLogger(TianyiClient.class);
    private static final String PROTOCOL = "https://api.ctyun.cn";
    private static final String batchGetMetricDataUrl = "/apiproxy/v3/batchGetMetricData";
    private static final String queryVmsUrl = "/apiproxy/v3/ondemand/queryVMs";
    private static final String queryMetricsUrl = "/apiproxy/v3/queryMetrics";
    private static final Integer SUCCESS_STATUS_CODE = 800;

    private final String platform = "3";

    private HttpClient client = new HttpClient(PostDataFormat.RAW);
    private String regionId;
    private String accessKey;
    private String secretKey;
    private String projectId;

    public final static Map<String, String> sysMetricMap = new HashMap();
    public final static Map<String, String> agentMetricMap = new HashMap();

    static {
        //基础监控
        sysMetricMap.put("cpu_util", "CPU使用率");
        sysMetricMap.put("mem_util", "内存使用率");
        sysMetricMap.put("disk_util_inband", "磁盘使用率");
        sysMetricMap.put("disk_read_bytes_rate", "磁盘读速率");
        sysMetricMap.put("disk_write_bytes_rate", "磁盘写速率");
        sysMetricMap.put("disk_read_requests_rate", "磁盘读操作速率");
        sysMetricMap.put("disk_write_requests_rate", "磁盘写操作速率");
        sysMetricMap.put("network_incoming_bytes_rate_inband", "带内网络流入速率");
        sysMetricMap.put("network_outgoing_bytes_rate_inband", "内网络流出速率");
        sysMetricMap.put("network_incoming_bytes_aggregate_rate", "带外网络流入速率");
        sysMetricMap.put("network_outgoing_bytes_aggregate_rate", "带外网络流出速率");
        sysMetricMap.put("inst_sys_status_error", "系统状态检查失败");
        sysMetricMap.put("network_vm_connections", "网络连接数");

        //Agent
        agentMetricMap.put("id_99434da1075811a47b2e2381114d7a9d", "disk_usedPercent");
        agentMetricMap.put("id_1253b90bc432a597b5dfe9bc97543ace", "disk_used");
        agentMetricMap.put("id_87504fa222b138b32d3e42101fae688f", "disk_free");
        agentMetricMap.put("id_ecd18c5759fe913966d38d911cb81bfd", "disk_total");
        agentMetricMap.put("cpu_usage", "cpu_usage");
        agentMetricMap.put("mem_usedPercent", "mem_usedPercent");
    }

    public List<MetricWithValueModel> batchGetMetricDataById(Map<String, String> instanceIds) {
        List<MetricWithValueModel> res = new ArrayList<>();
        Map<String, Object> params = new HashMap<>();
        Map<String, Object> paramJson = new HashMap<>();

        params.put("regionId", regionId);
        params.put("projectId", projectId);
        LocalDateTime now = LocalDateTime.now();
        long to = now.toInstant(ZoneOffset.of("+8")).toEpochMilli();
        long from = now.minusMinutes(6).toInstant(ZoneOffset.of("+8")).toEpochMilli();
        paramJson.put("from", from);
        paramJson.put("to", to);
        paramJson.put("period", "1");
        paramJson.put("filter", "average");
        List<MetricModel> metrics = new ArrayList<>();

        instanceIds.keySet().stream().forEach(instanceId -> {
            for (String metricName : sysMetricMap.keySet()) {
                metrics.add(new MetricModel(metricName, "SYS.ECS", Arrays.asList(new Dimension("instance_id", instanceId))));
            }
            for (String metricName : agentMetricMap.keySet()) {
                metrics.add(new MetricModel(metricName, "AGT.ECS", Arrays.asList(new Dimension("instance_id", instanceId))));
            }
        });

        int size = metrics.size();
        int i = 0;
        while (size > 0) {
            size -= 10;
            List<MetricModel> metricModels = metrics.stream().skip(10 * i).limit(10).collect(Collectors.toList());
            i++;
            paramJson.put("metrics", metricModels);

            SerializeConfig config = new SerializeConfig();
            config.propertyNamingStrategy = PropertyNamingStrategy.SnakeCase;
            params.put("paramJson", JSON.toJSONString(paramJson, config));

            client.setDataFormat(PostDataFormat.FORM_URLENCODED);
            Result result = this.client.post(buildHeader(batchGetMetricDataUrl, null), params, buildUrl(batchGetMetricDataUrl));
            Assert.isTrue(result.isSuccess(), "batch get metrics data , error:" + result.getMessage());

            JSONObject object = JSONObject.parseObject(result.getMessage());
            if (!object.getInteger("statusCode").equals(SUCCESS_STATUS_CODE)) {
                logger.error(JSONObject.toJSONString(object));
                throw new IllegalArgumentException();
            }
            JSONObject data = object.getJSONObject("returnObj");
            res.addAll(data.getJSONArray("metrics").toJavaList(MetricWithValueModel.class));
        }
        return res;
    }

    public Map<String, String> getVms() {
        HashMap<String, String> res = new HashMap<>();
        Map<String, Object> params = new HashMap<>();
        params.put("regionId", regionId);
        params.put("projectId", projectId);
        Result result = this.client.get(buildHeader(queryVmsUrl, params), params, buildUrl(queryVmsUrl));
        Assert.isTrue(result.isSuccess(), "get vms failed , error:" + result.getMessage());
        JSONObject object = JSONObject.parseObject(result.getMessage());
        if (!object.getInteger("statusCode").equals(SUCCESS_STATUS_CODE)) {
            logger.error(JSONObject.toJSONString(object));
            throw new IllegalArgumentException();
        }
        JSONObject data = object.getJSONObject("returnObj");
        JSONArray servers = data.getJSONArray("servers");
        for (int i = 0; i < servers.size(); i++) {
            JSONObject o = (JSONObject) servers.get(i);
            res.put(o.get("id").toString(), o.get("name").toString());
        }
        return res;
    }

    /**
     * 查询指标项列表
     *
     * @return
     */
    public JSONArray queryMetrics() {
        Map<String, Object> params = new HashMap<>();
        Map<String, Object> queryMetricsInfo = new HashMap<>();
        queryMetricsInfo.put("namespace", "AGT.ECS");
        queryMetricsInfo.put("regionId", regionId);
        params.put("queryMetricsInfo", JSONObject.toJSONString(queryMetricsInfo));
        Result result = this.client.get(buildHeader(queryMetricsUrl, params), params, buildUrl(queryMetricsUrl));
        Assert.isTrue(result.isSuccess(), "get vms failed , error:" + result.getMessage());
        JSONObject object = JSONObject.parseObject(result.getMessage());
        if (!object.getInteger("statusCode").equals(SUCCESS_STATUS_CODE)) {
            logger.error(JSONObject.toJSONString(object));
            throw new IllegalArgumentException();
        }
        JSONObject data = object.getJSONObject("returnObj");
        return data.getJSONArray("metrics");
    }

    private Map<String, Object> buildHeader(String url, Map<String, Object> headers) {
        Map<String, Object> header = new HashMap<>();
        header.put("Content-type", "application/x-www-form-urlencoded");
        Map<String, Object> getHeader = Util.sendGet(url, headers, this.accessKey, this.secretKey, this.platform);
        header.putAll(getHeader);
        if (!MapTools.isEmpty(headers)) {
            header.putAll(headers);
        }
        return header;
    }

    private String buildUrl(String apiUrl) {
        return PROTOCOL + apiUrl;
    }

}
