package com.bocloud.mcs.collector.vmware;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.encrypt.DESEncrptor;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.utils.Common;
import com.bocloud.ims.entity.CloudVendor;
import com.bocloud.ims.enums.CloudProvider;
import com.bocloud.mcs.collector.BocloudCollector;
import com.bocloud.service.core.CurrentService;
import com.bocloud.vmware.vim25.GuestDiskInfo;
import com.bocloud.vmware.vim25.VirtualMachineConnectionState;
import com.bocloud.vmware.vim25.mo.InventoryNavigator;
import com.bocloud.vmware.vim25.mo.ManagedEntity;
import com.bocloud.vmware.vim25.mo.ServiceInstance;
import com.bocloud.vmware.vim25.mo.VirtualMachine;
import com.google.common.util.concurrent.ThreadFactoryBuilder;
import io.prometheus.client.Collector;
import io.prometheus.client.GaugeMetricFamily;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.scheduling.annotation.Async;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;
import org.springframework.util.Assert;

import java.net.URL;
import java.util.*;
import java.util.concurrent.*;

/**
 * Vmware Partition collector
 *
 * @author zhangyf
 * @since 2021-02-01
 */
@Component
public class VcenterVmCollector extends BocloudCollector {

    private static final Logger logger = LoggerFactory.getLogger(VcenterVmCollector.class);

    ThreadFactory nameThreadFactory = new ThreadFactoryBuilder().setNameFormat("partition-pool-%d").build();
    private ExecutorService executorService = new ThreadPoolExecutor(Runtime.getRuntime().availableProcessors() + 1, Runtime.getRuntime().availableProcessors() + 1,
            0L, TimeUnit.MILLISECONDS,
            new LinkedBlockingQueue<>(), nameThreadFactory);

    Map<String, Object> cache = new HashMap<>();

    @Autowired
    private CurrentService currentService;

    @Override
    protected Map<String, Object> getCache() {
        return this.cache;
    }

    @Value("${collector.vmware.partition.enable:true}")
    boolean enable;

    /**
     * 缓存过期时间，30分钟
     *
     * @return
     */
    @Override
    public int getExpire() {
        return 1800000;
    }

    @Scheduled(initialDelay = 5000L, fixedRate = 900000L)
    @Async
    @Override
    public void scheduledCollect() {
        if (!currentService.isLeader()) {
            logger.info(this.getClass().getName() + "当前节点是否是主节点:" + currentService.isLeader());
            return;
        }
        Date now = new Date();
        logger.info("collect vmware partition metrics start...");
        BocloudResult bocloudResult = cmpCloudVendorService.list(CloudProvider.VMWARE);
        if (bocloudResult.isFailed()) {
            logger.error("get cloud vendor list failed , error:{}", bocloudResult.getMessage());
            return;
        }
        JSONObject data = (JSONObject) bocloudResult.getData();
        List<CloudVendor> listCloudVendor = data.getJSONArray(Common.ROWS).toJavaList(CloudVendor.class);
        List<String> labels = Arrays.asList("vmname", "vcenter", "path","uuid");

        GaugeMetricFamily collectTimeUse = new GaugeMetricFamily("vsphere_partition_collect_time_use", "help", Collections.singletonList("type"));

        List<Future<List<MetricFamilySamples>>> futures = new ArrayList<>();

        for (CloudVendor cloudVendor : listCloudVendor) {
            Future<List<Collector.MetricFamilySamples>> future = executorService.submit(() -> {
                String address = JSONObject.parseObject(cloudVendor.getAuthentication()).getString("address");
                String account = JSONObject.parseObject(cloudVendor.getAuthentication()).getString("account");
                String password = JSONObject.parseObject(cloudVendor.getAuthentication()).getString("password");
                password = new DESEncrptor().decrypt(password.trim(), "BocloudCMP");
                Assert.notNull(address, "VCenter EndPoint is Empty!");
                GaugeMetricFamily capacity = new GaugeMetricFamily("vsphere_partition_disk_capacity", "help", labels);
                GaugeMetricFamily freeSpace = new GaugeMetricFamily("vsphere_partition_disk_freeSpace", "help", labels);
                GaugeMetricFamily usedPercent = new GaugeMetricFamily("vsphere_partition_disk_usedPercent", "help", labels);
                GaugeMetricFamily committed = new GaugeMetricFamily("vsphere_vm_storage_committed", "help", Arrays.asList("vmname", "vcenter", "uuid"));
                GaugeMetricFamily uncommitted = new GaugeMetricFamily("vsphere_vm_storage_uncommitted", "help", Arrays.asList("vmname", "vcenter", "uuid"));
                GaugeMetricFamily unshared = new GaugeMetricFamily("vsphere_vm_storage_unshared", "help", Arrays.asList("vmname", "vcenter", "uuid"));
                GaugeMetricFamily provisioned = new GaugeMetricFamily("vsphere_vm_storage_provisioned", "help", Arrays.asList("vmname", "vcenter", "uuid"));
                URL url;
                if (address.contains(":")) {
                    String[] endpoints = address.split(":");
                    Assert.isTrue(Integer.parseInt(endpoints[1]) > 0, "VCenter EndPoint has Wrong Port");
                    url = new URL("https", endpoints[0], Integer.parseInt(endpoints[1]), "/sdk");
                } else {
                    url = new URL("https", address, "/sdk");
                }
                ServiceInstance serviceInstance = new ServiceInstance(url, account, password, true);
                ManagedEntity[] vms = new InventoryNavigator(serviceInstance.getRootFolder()).searchManagedEntities("VirtualMachine");
                for (ManagedEntity vm : vms) {
                    VirtualMachine machine = (VirtualMachine) vm;
                    GuestDiskInfo[] disk = machine.getGuest().getDisk();
                    if (disk == null) {
                        continue;
                    }
                    Double capacityTotal = 0d;
                    Double freeSpaceTotal = 0d;
                    String uuid = machine.getSummary().getConfig().getUuid();
                    // 机器异常时会获取不到uuid
                    if (StringUtils.isBlank(uuid)) {
                        continue;
                    }
                    committed.addMetric(Arrays.asList(machine.getName(), address, uuid), machine.getSummary().getStorage().getCommitted());
                    uncommitted.addMetric(Arrays.asList(machine.getName(), address, uuid), machine.getSummary().getStorage().getUncommitted());
                    unshared.addMetric(Arrays.asList(machine.getName(), address, uuid), machine.getSummary().getStorage().getUnshared());
                    long provisionedValue = machine.getSummary().getStorage().getCommitted() + machine.getSummary().getStorage().getUncommitted();
                    provisioned.addMetric(Arrays.asList(machine.getName(), address, uuid), provisionedValue);
                    // 磁盘监控信息
                    for (GuestDiskInfo diskInfo : disk) {
                        capacityTotal += diskInfo.getCapacity();
                        freeSpaceTotal += diskInfo.getFreeSpace();
                        capacity.addMetric(Arrays.asList(machine.getName(), address, diskInfo.getDiskPath(), uuid), diskInfo.getCapacity());
                        freeSpace.addMetric(Arrays.asList(machine.getName(), address, diskInfo.getDiskPath(), uuid), diskInfo.getFreeSpace());
                        Double usedPercentValue = diskInfo.getCapacity() == 0 ? 0 : (diskInfo.getCapacity() - diskInfo.getFreeSpace()) / Double.valueOf(String.valueOf(diskInfo.getCapacity())) * 100;
                        usedPercent.addMetric(Arrays.asList(machine.getName(), address, diskInfo.getDiskPath(), uuid), usedPercentValue);
                    }
                    usedPercent.addMetric(Arrays.asList(machine.getName(), address, "", uuid), capacityTotal == 0 ? 0 : (capacityTotal - freeSpaceTotal) / capacityTotal * 100);
                }
                List<Collector.MetricFamilySamples> mfs = new ArrayList<>();
                mfs.add(capacity);
                mfs.add(freeSpace);
                mfs.add(usedPercent);
                mfs.add(committed);
                mfs.add(uncommitted);
                mfs.add(unshared);
                mfs.add(provisioned);
                return mfs;
            });
            futures.add(future);
        }
        List<MetricFamilySamples> mfs = new ArrayList<>();
        Optional.ofNullable(futures).orElse(new ArrayList<>()).forEach(stringFuture -> {
            try {
                mfs.addAll(stringFuture.get());
            } catch (Exception e) {
                logger.error("collect vmware partition metrics error!" + e.getMessage(), e);
            }
        });
        collectTimeUse.addMetric(Arrays.asList("partition"), (System.currentTimeMillis() - now.getTime()));
        mfs.add(collectTimeUse);
        cache.put(TIME, new Date());
        cache.put(DATA, mfs);
        logger.info("collect vmware partition metrics end! use " + (System.currentTimeMillis() - now.getTime()) / 1000 + "s");
    }
}
