package com.bocloud.mcs.common;

import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.utils.NicTools;

import java.lang.management.ClassLoadingMXBean;
import java.lang.management.ManagementFactory;
import java.lang.management.MemoryMXBean;
import java.lang.management.OperatingSystemMXBean;
import java.lang.management.RuntimeMXBean;
import java.lang.management.ThreadMXBean;
import java.math.BigDecimal;
import java.text.DecimalFormat;
import java.text.DecimalFormatSymbols;
import java.text.NumberFormat;
import java.util.HashMap;
import java.util.Locale;
import java.util.Map;

public class SystemTool {
    private final NumberFormat fmtI;
    private final NumberFormat fmtD;

    public SystemTool() {
        this.fmtI = new DecimalFormat("###,###", new DecimalFormatSymbols(Locale.ENGLISH));
        this.fmtD = new DecimalFormat("###,##0.000", new DecimalFormatSymbols(Locale.ENGLISH));
    }

    public BocloudResult<Map<String, Object>> status() {
        Map<String, Object> state = new HashMap<>(16, 0.75F);
        OperatingSystemMXBean os = ManagementFactory.getOperatingSystemMXBean();
        RuntimeMXBean runtime = ManagementFactory.getRuntimeMXBean();
        ThreadMXBean threads = ManagementFactory.getThreadMXBean();
        MemoryMXBean memory = ManagementFactory.getMemoryMXBean();
        ClassLoadingMXBean classloader = ManagementFactory.getClassLoadingMXBean();
        long memoryUsed = memory.getHeapMemoryUsage().getUsed() + memory.getNonHeapMemoryUsage().getUsed();
        state.put("os.user", System.getProperty("user.name"));
        state.put("os.name", os.getName() + "-" + os.getVersion());
        state.put("os.arch", os.getArch());
        state.put("os.cpu", Runtime.getRuntime().availableProcessors());
        state.put("os.ip", NicTools.getIPList().toArray());
        state.put("jvm.name", runtime.getVmName() + " " + runtime.getVmVersion());
        state.put("jvm.vendor", runtime.getVmVendor());
        state.put("jvm.version", System.getProperty("java.version"));
        state.put("jvm.uptime", this.toDuration((double)runtime.getUptime()));
        long MB = 1048576L;
        state.put("jvm.memory.used", memoryUsed / MB);
        state.put("jvm.memory.total", Runtime.getRuntime().totalMemory() / MB);
        state.put("jvm.memory.free", Runtime.getRuntime().freeMemory() / MB);
        state.put("jvm.memory.max", Runtime.getRuntime().maxMemory() / MB);
        state.put("jvm.heap.current", memory.getHeapMemoryUsage().getUsed() / MB);
        state.put("jvm.heap.max", memory.getHeapMemoryUsage().getMax() / MB);
        state.put("jvm.heap.committed", memory.getHeapMemoryUsage().getCommitted() / MB);
        state.put("jvm.threads.live", threads.getThreadCount());
        state.put("jvm.threads.daemon", threads.getDaemonThreadCount());
        state.put("jvm.threads.peak", threads.getPeakThreadCount());
        state.put("jvm.threads.total", threads.getTotalStartedThreadCount());
        state.put("jvm.class.unload", classloader.getUnloadedClassCount());
        state.put("jvm.class.total", classloader.getTotalLoadedClassCount());
        state.put("jvm.class.current", classloader.getLoadedClassCount());
        state.put("jvm.cpu.usage", this.getCpuUsage());
        return new BocloudResult(true, state, "ok");
    }

    protected String printSizeInKb(double size) {
        return this.fmtI.format((long)(size / 1024.0D)) + " kbytes";
    }

    protected String toDuration(double uptime) {
        uptime /= 1000.0D;
        if (uptime < 60.0D) {
            return this.fmtD.format(uptime) + " seconds";
        } else {
            uptime /= 60.0D;
            long days;
            if (uptime < 60.0D) {
                days = (long)uptime;
                return this.fmtI.format(days) + (days > 1L ? " minutes" : " minute");
            } else {
                uptime /= 60.0D;
                long hours;
                String s;
                if (uptime < 24.0D) {
                    days = (long)uptime;
                    hours = (long)((uptime - (double)days) * 60.0D);
                    s = this.fmtI.format(days) + (days > 1L ? " hours" : " hour");
                    if (hours != 0L) {
                        s = s + " " + this.fmtI.format(hours) + (hours > 1L ? " minutes" : " minute");
                    }

                    return s;
                } else {
                    uptime /= 24.0D;
                    days = (long)uptime;
                    hours = (long)((uptime - (double)days) * 24.0D);
                    s = this.fmtI.format(days) + (days > 1L ? " days" : " day");
                    if (hours != 0L) {
                        s = s + " " + this.fmtI.format(hours) + (hours > 1L ? " hours" : " hour");
                    }

                    return s;
                }
            }
        }
    }

    private float getCpuUsage() {
        long jvmStart = 0L;
        long jvmStop = 0L;
        long osStart = ManagementFactory.getRuntimeMXBean().getUptime();
        OperatingSystemMXBean osMXBean = ManagementFactory.getOperatingSystemMXBean();
        com.sun.management.OperatingSystemMXBean sunOSMXBean;
        if (osMXBean instanceof com.sun.management.OperatingSystemMXBean) {
            sunOSMXBean = (com.sun.management.OperatingSystemMXBean)osMXBean;
            jvmStart = sunOSMXBean.getProcessCpuTime();
        }

        try {
            Thread.sleep(3000L);
        } catch (InterruptedException var13) {
        }

        long osStop = ManagementFactory.getRuntimeMXBean().getUptime();
        osMXBean = ManagementFactory.getOperatingSystemMXBean();
        if (osMXBean instanceof com.sun.management.OperatingSystemMXBean) {
            sunOSMXBean = (com.sun.management.OperatingSystemMXBean)osMXBean;
            jvmStop = sunOSMXBean.getProcessCpuTime();
        }

        BigDecimal jvmCpuUsed = BigDecimal.valueOf(jvmStop - jvmStart).divide(BigDecimal.valueOf(1000000.0D), 4, 4);
        BigDecimal osCpuUsed = BigDecimal.valueOf(osStop - osStart);
        BigDecimal cpuUsage = jvmCpuUsed.divide(osCpuUsed, 4, 4);
        return cpuUsage.floatValue();
    }
}
